import time
from xml.etree.ElementTree import Element, SubElement
from docmaptools import parse as docmap_parse
from jatsgenerator import build as jats_build
from elifetools.utils import doi_to_doi_uri
from elifecleaner import sub_article, LOGGER
from elifecleaner.prc import date_struct_from_string


def prune_history_data(history_data, doi, version):
    "return history data related to doi for versions less than version provided"
    return [
        data
        for data in history_data
        if not data.get("doi").startswith(doi)
        or (
            data.get("doi").startswith(doi)
            and data.get("versionIdentifier")
            and int(data.get("versionIdentifier")) < int(version)
        )
    ]


def find_pub_history_tag(root, identifier=None):
    "find pub-history tag in the article-meta tag"
    article_meta_tag = root.find(".//front/article-meta")
    if article_meta_tag is None:
        LOGGER.warning(
            "%s article-meta tag not found",
            identifier,
        )
        return None
    # look for an existing pub-history tag
    pub_history_tag = article_meta_tag.find("./pub-history")
    # if no pub-history tag, add one
    if pub_history_tag is None:
        # insert the new tag into the XML after the history or elocation-id tag
        insert_index = 1
        for tag_index, tag in enumerate(article_meta_tag.findall("*")):
            if tag.tag in "history":
                insert_index = tag_index + 1
                break
            if tag.tag == "elocation-id":
                insert_index = tag_index + 1
        pub_history_tag = Element("pub-history")
        article_meta_tag.insert(insert_index, pub_history_tag)
    return pub_history_tag


def add_self_uri_tag(parent, article_type, uri, title):
    "add self-uri tag for each peer review in event tag"
    self_uri_tag = SubElement(parent, "self-uri")
    if article_type:
        self_uri_tag.set("content-type", article_type)
    self_uri_tag.set(
        "{http://www.w3.org/1999/xlink}href",
        uri,
    )
    self_uri_tag.text = str(title)


def add_history_event_tag(parent, event_data):
    "add event tag to pub-history"
    event_tag = SubElement(parent, "event")
    if event_data.get("event_desc"):
        event_desc_tag = SubElement(event_tag, "event-desc")
        event_desc_tag.text = event_data.get("event_desc")
    if event_data.get("date"):
        date_struct = date_struct_from_string(event_data.get("date"))
        date_tag = SubElement(event_tag, "date")
        if event_data.get("type"):
            date_tag.set("date-type", event_data.get("type"))
        jats_build.set_dmy(date_tag, date_struct)
        date_tag.set("iso-8601-date", time.strftime("%Y-%m-%d", date_struct))
        if event_data.get("doi"):
            self_uri_tag = SubElement(event_tag, "self-uri")
            if event_data.get("type"):
                self_uri_tag.set("content-type", event_data.get("type"))
            self_uri_tag.set(
                "{http://www.w3.org/1999/xlink}href",
                doi_to_doi_uri(event_data.get("doi")),
            )
    if event_data.get("self_uri_list"):
        for self_uri_data in event_data.get("self_uri_list"):
            add_self_uri_tag(
                event_tag,
                self_uri_data.get("content_type"),
                self_uri_data.get("uri"),
                self_uri_data.get("title"),
            )


EVENT_DESC_PREPRINT = "This manuscript was published as a preprint."
EVENT_DESC_REVIEWED_PREPRINT = "This manuscript was published as a reviewed preprint."
EVENT_DESC_REVISED_PREPRINT = "The reviewed preprint was revised."

MECA_EVENT_DESC_PREPRINT = "Preprint posted"
MECA_EVENT_DESC_REVIEWED_PREPRINT = "Reviewed preprint"
MECA_EVENT_DESC_REVISED_PREPRINT = "Reviewed preprint"


def preprint_event_desc(style):
    "generate event event-desc tag text for a preprint history event"
    event_description = None
    if style == "accepted":
        event_description = EVENT_DESC_PREPRINT
    elif style == "meca":
        event_description = MECA_EVENT_DESC_PREPRINT
    return event_description


def reviewed_preprint_event_desc(style, first_review_event, version):
    "generate event event-desc tag text for a reviewed preprint history event"
    event_description = None
    extra = ""
    if style == "meca" and version:
        extra = " v%s" % version
    if first_review_event:
        # adding the first reviewed-preprint
        if style == "accepted":
            event_description = "%s%s" % (
                EVENT_DESC_REVIEWED_PREPRINT,
                extra,
            )
        elif style == "meca":
            event_description = "%s%s" % (
                MECA_EVENT_DESC_REVIEWED_PREPRINT,
                extra,
            )
    else:
        # already added one reviewed-preprint
        if style == "accepted":
            event_description = "%s%s" % (
                EVENT_DESC_REVISED_PREPRINT,
                extra,
            )
        elif style == "meca":
            event_description = "%s%s" % (
                MECA_EVENT_DESC_REVISED_PREPRINT,
                extra,
            )
    return event_description


def history_event_self_uri_list(docmap_string, version_doi):
    "a list of self-uri tag data for a history event"
    self_uri_list = []
    d_json = docmap_parse.docmap_json(docmap_string)
    step_map = docmap_parse.preprint_version_doi_step_map(d_json)
    if step_map.get(version_doi):
        generate_dois = False
        data = sub_article.sub_article_data(
            docmap_string,
            version_doi=version_doi,
            generate_dois=generate_dois,
        )
        for data_item in data:
            self_uri_data = {}
            if data_item.get("article").article_type:
                self_uri_data["content_type"] = data_item.get("article").article_type
            self_uri_data["uri"] = doi_to_doi_uri(data_item.get("article").doi)
            self_uri_data["title"] = str(data_item.get("article").title)
            self_uri_list.append(self_uri_data)
    return self_uri_list


def collect_history_event_data(
    history_data, style, docmap_string=None, add_self_uri=False
):
    "populate a list of history event data"
    history_event_data = []
    if not history_data:
        return history_event_data
    first_review_event = True
    for data in history_data:
        event_data = {}
        event_data["type"] = data.get("type")
        event_data["date"] = data.get("date")
        event_data["doi"] = data.get("doi")
        event_data["versionIdentifier"] = data.get("versionIdentifier")
        # event description
        if event_data.get("type") == "preprint":
            event_data["event_desc"] = preprint_event_desc(style)
        elif event_data.get("type") == "reviewed-preprint":
            event_data["event_desc"] = reviewed_preprint_event_desc(
                style, first_review_event, event_data.get("versionIdentifier")
            )
            first_review_event = False
        # self-uri values
        if add_self_uri and docmap_string:
            event_data["self_uri_list"] = history_event_self_uri_list(
                docmap_string, version_doi=event_data.get("doi")
            )
        # append the event data to the list
        history_event_data.append(event_data)
    return history_event_data


def add_pub_history_tag(root, history_event_data, identifier=None):
    "set event data in pub-history tag"
    if not history_event_data:
        return root
    pub_history_tag = find_pub_history_tag(root, identifier)
    if pub_history_tag is None:
        return root
    # add event tags to the pub-history tag
    for event_data in history_event_data:
        add_history_event_tag(
            pub_history_tag,
            event_data,
        )
    return root


def add_pub_history(root, history_data, docmap_string=None, identifier=None):
    "add the pub-history tag and add event data to it"
    style = "accepted"
    add_self_uri = False
    # collect history event data
    history_event_data = collect_history_event_data(
        history_data, style, docmap_string, add_self_uri
    )
    return add_pub_history_tag(root, history_event_data, identifier)


def add_pub_history_meca(root, history_data, docmap_string=None, identifier=None):
    "add the MECA style pub-history tag and add event data to it"
    style = "meca"
    add_self_uri = True
    # collect history event data
    history_event_data = collect_history_event_data(
        history_data, style, docmap_string, add_self_uri
    )
    return add_pub_history_tag(root, history_event_data, identifier)



import os
import time
import unittest
from xml.etree import ElementTree
import json
from elifearticle.article import Affiliation, Contributor, Role
from elifetools import xmlio
from elifecleaner import LOGGER, configure_logging, prc
from tests.helpers import delete_files_in_folder, read_fixture


# elife ISSN example of non-PRC journal-id tag values
NON_PRC_XML = (
    "<article><front><journal-meta>"
    '<journal-id journal-id-type="nlm-ta">elife</journal-id>'
    '<journal-id journal-id-type="hwp">eLife</journal-id>'
    '<journal-id journal-id-type="publisher-id">eLife</journal-id>'
    "<journal-title-group>"
    "<journal-title>eLife</journal-title>"
    "</journal-title-group>"
    '<issn pub-type="epub">2050-084X</issn>'
    "<publisher>"
    "<publisher-name>eLife Sciences Publications, Ltd</publisher-name>"
    "</publisher>"
    "</journal-meta></front></article>"
)

# PRC xml will have non-eLife journal-id tag text values
PRC_XML = (
    "<article><front><journal-meta>"
    '<journal-id journal-id-type="nlm-ta">foo</journal-id>'
    '<journal-id journal-id-type="hwp">foo</journal-id>'
    '<journal-id journal-id-type="publisher-id">foo</journal-id>'
    "<journal-title-group>"
    "<journal-title>eLife Reviewed Preprints </journal-title>"
    "</journal-title-group>"
    '<issn pub-type="epub">2050-084X</issn>'
    "<publisher>"
    "<publisher-name>elife-rp Sciences Publications, Ltd</publisher-name>"
    "</publisher>"
    "</journal-meta></front></article>"
)


class TestIsXmlPrc(unittest.TestCase):
    def test_is_xml_prc(self):
        "PRC XML will return true"
        root = ElementTree.fromstring(PRC_XML)
        self.assertTrue(prc.is_xml_prc(root))

    def test_is_xml_prc_false(self):
        "test non-PRC XML will return false"
        root = ElementTree.fromstring(NON_PRC_XML)
        self.assertEqual(prc.is_xml_prc(root), False)

    def test_is_xml_prc_incomplete(self):
        "incomplete XML will return false"
        root = ElementTree.fromstring("<root/>")
        self.assertEqual(prc.is_xml_prc(root), False)

    def test_is_xml_prc_elocation_id(self):
        "elocation-id value has already been changed on a PRC XML"
        root = ElementTree.fromstring(
            "<article>"
            "<front>"
            "<article-meta>"
            "<elocation-id>RP88273</elocation-id>"
            "</article-meta>"
            "</front>"
            "</article>"
        )
        self.assertEqual(prc.is_xml_prc(root), True)


class TestTransformJournalIdTags(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_transform_journal_id_tags(self):
        # populate an ElementTree
        identifier = "test.zip"
        xml_string = PRC_XML
        root = ElementTree.fromstring(xml_string)
        # invoke the function
        root_output = prc.transform_journal_id_tags(root, identifier)
        # assertions
        self.assertTrue(
            b'<journal-id journal-id-type="nlm-ta">elife</journal-id>'
            in ElementTree.tostring(root_output)
        )
        self.assertTrue(
            b'<journal-id journal-id-type="hwp">eLife</journal-id>'
            in ElementTree.tostring(root_output)
        )
        self.assertTrue(
            b'<journal-id journal-id-type="publisher-id">eLife</journal-id>'
            in ElementTree.tostring(root_output)
        )

        log_file_lines = []
        with open(self.log_file, "r") as open_file:
            for line in open_file:
                log_file_lines.append(line)
        for index, (journal_id_type, tag_text) in enumerate(
            [("nlm-ta", "elife"), ("hwp", "eLife"), ("publisher-id", "eLife")]
        ):
            self.assertEqual(
                log_file_lines[index],
                (
                    (
                        "INFO elifecleaner:prc:transform_journal_id_tags: "
                        "%s replacing journal-id tag text of type %s to %s\n"
                    )
                )
                % (identifier, journal_id_type, tag_text),
            )


class TestTransformJournalTitleTag(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_journal_title(self):
        # populate an ElementTree
        identifier = "test.zip"
        xml_string = PRC_XML
        tag_text = "eLife"
        root = ElementTree.fromstring(xml_string)
        # invoke the function
        root_output = prc.transform_journal_title_tag(root, identifier)
        # assertions
        self.assertTrue(
            b"<journal-title>%s</journal-title>" % bytes(tag_text, encoding="utf-8")
            in ElementTree.tostring(root_output)
        )
        log_file_lines = []
        with open(self.log_file, "r") as open_file:
            for line in open_file:
                log_file_lines.append(line)
        self.assertEqual(
            log_file_lines[-1],
            (
                (
                    "INFO elifecleaner:prc:transform_journal_meta_tag: "
                    "%s replacing journal-title tag text to %s\n"
                )
            )
            % (identifier, tag_text),
        )


class TestTransformPublisherNameTag(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_publisher_name(self):
        # populate an ElementTree
        identifier = "test.zip"
        xml_string = PRC_XML
        tag_text = "eLife Sciences Publications, Ltd"
        root = ElementTree.fromstring(xml_string)
        # invoke the function
        root_output = prc.transform_publisher_name_tag(root, identifier)
        # assertions
        self.assertTrue(
            b"<publisher-name>%s</publisher-name>" % bytes(tag_text, encoding="utf-8")
            in ElementTree.tostring(root_output)
        )
        log_file_lines = []
        with open(self.log_file, "r") as open_file:
            for line in open_file:
                log_file_lines.append(line)
        self.assertEqual(
            log_file_lines[-1],
            (
                (
                    "INFO elifecleaner:prc:transform_journal_meta_tag: "
                    "%s replacing publisher-name tag text to %s\n"
                )
            )
            % (identifier, tag_text),
        )


class TestAddPrcCustomMetaTags(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)
        self.expected_xml = bytes(
            "<article>"
            "<front>"
            "<article-meta>"
            "<custom-meta-group>"
            '<custom-meta specific-use="meta-only">'
            "<meta-name>publishing-route</meta-name>"
            "<meta-value>prc</meta-value>"
            "</custom-meta>"
            "</custom-meta-group>"
            "</article-meta>"
            "</front>"
            "</article>",
            encoding="utf-8",
        )

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_add_custom_meta_tags(self):
        "test when custom-meta-group tag does not yet exist"
        # populate an ElementTree
        xml_string = "<article><front><article-meta/></front></article>"
        root = ElementTree.fromstring(xml_string)
        # invoke the function
        root_output = prc.add_prc_custom_meta_tags(root)
        # assertions
        self.assertEqual(ElementTree.tostring(root_output), self.expected_xml)

    def test_group_tag_exists(self):
        "test if custom-meta-group tag already exists"
        # populate an ElementTree
        xml_string = (
            "<article><front><article-meta>"
            "<custom-meta-group />"
            "</article-meta></front></article>"
        )
        root = ElementTree.fromstring(xml_string)
        # invoke the function
        root_output = prc.add_prc_custom_meta_tags(root)
        # assertions
        self.assertEqual(ElementTree.tostring(root_output), self.expected_xml)

    def test_no_article_meta_tag(self):
        # populate an ElementTree
        identifier = "test.zip"
        xml_string = "<root/>"
        expected = b"<root />"
        root = ElementTree.fromstring(xml_string)
        # invoke the function
        root_output = prc.add_prc_custom_meta_tags(root, identifier)
        # assertions
        self.assertEqual(ElementTree.tostring(root_output), expected)
        with open(self.log_file, "r") as open_file:
            self.assertEqual(
                open_file.read(),
                (
                    "WARNING elifecleaner:prc:add_prc_custom_meta_tags: "
                    "%s article-meta tag not found\n"
                )
                % identifier,
            )


class TestElocationIdFromDocmap(unittest.TestCase):
    "tests for prc.elocation_id_from_docmap()"

    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)
        self.identifier = "test.zip"

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_elocation_id_from_docmap(self):
        "get elocation-id from docmap data"
        docmap_json = docmap_test_data()
        expected = "RP85111"
        # invoke
        elocation_id = prc.elocation_id_from_docmap(
            json.dumps(docmap_json), identifier=self.identifier
        )
        # assert
        self.assertEqual(elocation_id, expected)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
        self.assertEqual(
            log_messages[0],
            ("INFO elifecleaner:prc:elocation_id_from_docmap: " "Parse docmap json\n"),
        )

    def test_find_by_version_doi(self):
        "get elocation-id from docmap data by matching version DOI value"
        docmap_json = docmap_test_data()
        version_doi = "10.7554/eLife.85111.1"
        expected = "RP85111"
        # invoke
        elocation_id = prc.elocation_id_from_docmap(
            json.dumps(docmap_json), version_doi=version_doi, identifier=self.identifier
        )
        # assert
        self.assertEqual(elocation_id, expected)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
        self.assertEqual(
            log_messages[0],
            ("INFO elifecleaner:prc:elocation_id_from_docmap: " "Parse docmap json\n"),
        )

    def test_no_history_data(self):
        "test if no history data can be found in the docmap"
        docmap_json = {"first-step": "_:b0", "steps": {"_:b0": {}}}
        expected = None
        # invoke
        elocation_id = prc.elocation_id_from_docmap(
            json.dumps(docmap_json), identifier=self.identifier
        )
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
        # assert
        self.assertEqual(elocation_id, expected)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[-1],
                (
                    "WARNING elifecleaner:prc:elocation_id_from_docmap: "
                    "%s no elocation_id found in the docmap\n"
                )
                % self.identifier,
            )

    def test_docmap_is_none(self):
        "test if docmap is empty"
        docmap_json = {}
        # invoke
        elocation_id = prc.elocation_id_from_docmap(
            json.dumps(docmap_json), identifier=self.identifier
        )
        # assert
        self.assertEqual(elocation_id, None)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[-1],
                (
                    "WARNING elifecleaner:prc:elocation_id_from_docmap: "
                    "%s parsing docmap returned None\n"
                )
                % self.identifier,
            )


def docmap_test_data(doi=None):
    "generate a docmap json test fixture"
    docmap_json = {
        "first-step": "_:b0",
        "steps": {
            "_:b0": {
                "actions": [
                    {
                        "participants": [],
                        "outputs": [
                            {
                                "type": "preprint",
                                "identifier": "85111",
                                "doi": "10.7554/eLife.85111.1",
                                "versionIdentifier": "1",
                                "license": "http://creativecommons.org/licenses/by/4.0/",
                            }
                        ],
                    }
                ],
                "assertions": [
                    {
                        "item": {
                            "type": "preprint",
                            "doi": "10.1101/2022.11.08.515698",
                            "versionIdentifier": "2",
                        },
                        "status": "under-review",
                        "happened": "2022-11-28T11:30:05+00:00",
                    }
                ],
                "next-step": "_:b1",
            },
            "_:b1": {
                "actions": [
                    {
                        "outputs": [
                            {
                                "type": "preprint",
                                "identifier": "85111",
                                "versionIdentifier": "1",
                                "license": "http://creativecommons.org/licenses/by/4.0/",
                                "published": "2023-01-25T14:00:00+00:00",
                                "partOf": {
                                    "type": "manuscript",
                                    "doi": "10.7554/eLife.85111",
                                    "identifier": "85111",
                                    "subjectDisciplines": ["Neuroscience"],
                                    "published": "2023-01-25T14:00:00+00:00",
                                    "volumeIdentifier": "12",
                                    "electronicArticleIdentifier": "RP85111",
                                },
                            }
                        ]
                    },
                ]
            },
        },
    }
    if doi:
        # add doi key and value to the outputs
        docmap_json["steps"]["_:b1"]["actions"][0]["outputs"][0]["doi"] = doi
    return docmap_json


class TestVersionDoiFromDocmap(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)
        self.identifier = "test.zip"

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_version_doi_from_docmap(self):
        "test for when a doi is found"
        doi = "10.7554/eLife.85111.2"
        docmap_json = docmap_test_data(doi)
        result = prc.version_doi_from_docmap(json.dumps(docmap_json), self.identifier)
        self.assertEqual(result, doi)

    def test_unpublished_version_doi(self):
        "test argument published is False"
        doi = "10.7554/eLife.85111.2"
        docmap_json = docmap_test_data(doi)
        # delete the published dict key
        del docmap_json["steps"]["_:b1"]["actions"][0]["outputs"][0]["published"]
        result = prc.version_doi_from_docmap(
            json.dumps(docmap_json), self.identifier, False
        )
        self.assertEqual(result, doi)

    def test_docmap_is_none(self):
        "test for no docmap"
        docmap_json = {}
        result = prc.version_doi_from_docmap(json.dumps(docmap_json), self.identifier)
        self.assertEqual(result, None)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[-1],
                (
                    "WARNING elifecleaner:prc:version_doi_from_docmap: "
                    "%s parsing docmap returned None\n"
                )
                % self.identifier,
            )

    def test_no_preprint_in_docmap(self):
        "test for doi is not present"
        docmap_json = docmap_test_data(None)
        # delete the step holding the preprint data
        del docmap_json["steps"]["_:b1"]
        result = prc.version_doi_from_docmap(json.dumps(docmap_json), self.identifier)
        self.assertEqual(result, None)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[-1],
                (
                    "WARNING elifecleaner:prc:version_doi_from_docmap: "
                    "%s no preprint data was found in the docmap\n"
                )
                % self.identifier,
            )

    def test_no_doi_key_in_docmap(self):
        "test for doi is not present"
        docmap_json = docmap_test_data(None)
        result = prc.version_doi_from_docmap(json.dumps(docmap_json), self.identifier)
        self.assertEqual(result, None)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[-1],
                (
                    "WARNING elifecleaner:prc:version_doi_from_docmap: "
                    "%s did not find doi data in the docmap preprint data\n"
                )
                % self.identifier,
            )


class TestNextVersionDoi(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)
        self.identifier = "test.zip"

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_next_version_doi(self):
        doi = "10.7554/eLife.85111.2"
        expected = "10.7554/eLife.85111.3"
        result = prc.next_version_doi(doi, self.identifier)
        self.assertEqual(result, expected)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[-1],
                (
                    "INFO elifecleaner:prc:next_version_doi: "
                    "%s next version doi, from DOI %s, next DOI %s\n"
                )
                % (self.identifier, doi, expected),
            )

    def test_non_int_version(self):
        "non-int version value at the end"
        version = "sa1"
        doi = "10.7554/eLife.85111.2.%s" % version
        expected = None
        result = prc.next_version_doi(doi, self.identifier)
        self.assertEqual(result, expected)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[-1],
                (
                    "WARNING elifecleaner:prc:next_version_doi: "
                    "%s version from DOI could not be converted to int, version %s\n"
                )
                % (self.identifier, version),
            )

    def test_version_exceeds_limit(self):
        "non-int version value at the end"
        article_id = "85111"
        doi = "10.7554/eLife.%s" % article_id
        expected = None
        result = prc.next_version_doi(doi, self.identifier)
        self.assertEqual(result, expected)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[-1],
                (
                    "WARNING elifecleaner:prc:next_version_doi: "
                    "%s failed to determine the version from DOI, "
                    "version %s exceeds MAX_VERSION %s\n"
                )
                % (self.identifier, article_id, prc.MAX_VERSION),
            )

    def test_none(self):
        "non-int version value at the end"
        doi = None
        expected = None
        result = prc.next_version_doi(doi, self.identifier)
        self.assertEqual(result, expected)


class TestAddArticleId(unittest.TestCase):
    "tests for add_article_id()"

    def test_add_article_id(self):
        "test adding concept DOI article-id tag"
        xml_string = "<article-meta />"
        root = ElementTree.fromstring(xml_string)
        doi = "10.7554/eLife.1234567890"
        pub_id_type = "doi"
        expected = (
            "<article-meta>"
            '<article-id pub-id-type="%s">%s</article-id>'
            "</article-meta>"
        ) % (pub_id_type, doi)
        prc.add_article_id(root, doi, pub_id_type)
        self.assertEqual(ElementTree.tostring(root).decode("utf-8"), expected)

    def test_version_doi(self):
        "test adding a version DOI article-id tag"
        xml_string = "<article-meta />"
        root = ElementTree.fromstring(xml_string)
        doi = "10.7554/eLife.1234567890"
        pub_id_type = "doi"
        specific_use = "version"
        expected = (
            "<article-meta>"
            '<article-id pub-id-type="%s" specific-use="%s">%s</article-id>'
            "</article-meta>"
        ) % (pub_id_type, specific_use, doi)
        prc.add_article_id(root, doi, pub_id_type, specific_use)
        self.assertEqual(ElementTree.tostring(root).decode("utf-8"), expected)


class TestAddVersionDoi(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)
        self.doi = "10.7554/eLife.1234567890.5"
        self.identifier = "test.zip"

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_add_version_doi(self):
        xml_string = "<article><front><article-meta /></front></article>"
        root = ElementTree.fromstring(xml_string)
        expected = (
            b"<article>"
            b"<front>"
            b"<article-meta>"
            b'<article-id pub-id-type="doi" specific-use="version">'
            b"10.7554/eLife.1234567890.5"
            b"</article-id>"
            b"</article-meta>"
            b"</front>"
            b"</article>"
        )
        root_output = prc.add_version_doi(root, self.doi, self.identifier)
        self.assertEqual(ElementTree.tostring(root_output), expected)

    def test_add_version_doi_in_order(self):
        "test the new article-id tag is added in a particular order"
        xml_string = (
            "<article>"
            "<front>"
            "<article-meta>"
            "<article-id />"
            "<open-access>YES</open-access>"
            "</article-meta>"
            "</front>"
            "</article>"
        )
        root = ElementTree.fromstring(xml_string)
        expected = (
            b"<article>"
            b"<front>"
            b"<article-meta>"
            b"<article-id />"
            b'<article-id pub-id-type="doi" specific-use="version">'
            b"10.7554/eLife.1234567890.5"
            b"</article-id>"
            b"<open-access>YES</open-access>"
            b"</article-meta>"
            b"</front>"
            b"</article>"
        )
        root_output = prc.add_version_doi(root, self.doi, self.identifier)
        self.assertEqual(ElementTree.tostring(root_output), expected)

    def test_no_article_meta(self):
        "test if no article-meta tag is in the XML"
        xml_string = "<article />"
        root = ElementTree.fromstring(xml_string)
        expected = bytes(xml_string, encoding="utf-8")
        root_output = prc.add_version_doi(root, self.doi, self.identifier)
        self.assertEqual(ElementTree.tostring(root_output), expected)
        with open(self.log_file, "r") as open_file:
            self.assertEqual(
                open_file.read(),
                ("WARNING elifecleaner:prc:add_doi: " "%s article-meta tag not found\n")
                % self.identifier,
            )


class TestReviewDateFromDocmap(unittest.TestCase):
    "tests for prc.review_date_from_docmap()"

    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)
        self.identifier = "test.zip"

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_review_date_from_docmap(self):
        "test docmap which has a review date"
        docmap_json = docmap_test_data()
        expected = "2022-11-28T11:30:05+00:00"
        # invoke
        date_string = prc.review_date_from_docmap(json.dumps(docmap_json))
        # assert
        self.assertEqual(date_string, expected)

    def test_no_assertions(self):
        "test docmap which has a review date"
        docmap_json = docmap_test_data()
        # remove assertions from the test data
        del docmap_json["steps"]["_:b0"]["assertions"]
        expected = None
        # invoke
        date_string = prc.review_date_from_docmap(json.dumps(docmap_json))
        # assert
        self.assertEqual(date_string, expected)

    def test_docmap_is_none(self):
        "test for no docmap"
        docmap_json = {}
        result = prc.review_date_from_docmap(json.dumps(docmap_json), self.identifier)
        self.assertEqual(result, None)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[-1],
                (
                    "WARNING elifecleaner:prc:review_date_from_docmap: "
                    "%s parsing docmap returned None\n"
                )
                % self.identifier,
            )


class TestArticleIdFromDocmap(unittest.TestCase):
    "tests for article_id_from_docmap()"

    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)
        self.identifier = "test.zip"

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_article_id_from_docmap(self):
        "parse article_id from docmap preprint output"
        docmap_json = docmap_test_data()
        expected = "85111"
        # invoke
        result = prc.article_id_from_docmap(
            json.dumps(docmap_json), identifier=self.identifier
        )
        # assert
        self.assertEqual(result, expected)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
        self.assertEqual(
            log_messages[0],
            ("INFO elifecleaner:prc:article_id_from_docmap: " "Parse docmap json\n"),
        )

    def test_no_article_id(self):
        "test if no article_id data can be found in the docmap"
        docmap_json = docmap_test_data()
        del docmap_json["steps"]["_:b1"]["actions"][0]["outputs"][0]["identifier"]
        expected = None
        # invoke
        result = prc.article_id_from_docmap(
            json.dumps(docmap_json), identifier=self.identifier
        )
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
        # assert
        self.assertEqual(result, expected)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[-1],
                (
                    "WARNING elifecleaner:prc:article_id_from_docmap: "
                    "%s no article_id found in the docmap\n"
                )
                % self.identifier,
            )

    def test_docmap_is_none(self):
        "test if docmap is empty"
        docmap_json = {}
        # invoke
        result = prc.article_id_from_docmap(
            json.dumps(docmap_json), identifier=self.identifier
        )
        # assert
        self.assertEqual(result, None)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[-1],
                (
                    "WARNING elifecleaner:prc:article_id_from_docmap: "
                    "%s parsing docmap returned None\n"
                )
                % self.identifier,
            )


class TestLicenseFromDocmap(unittest.TestCase):
    "tests for license_from_docmap()"

    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)
        self.identifier = "test.zip"

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_license_from_docmap(self):
        "parse license from docmap preprint output"
        docmap_json = docmap_test_data()
        expected = "http://creativecommons.org/licenses/by/4.0/"
        # invoke
        result = prc.license_from_docmap(
            json.dumps(docmap_json), identifier=self.identifier
        )
        # assert
        self.assertEqual(result, expected)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
        self.assertEqual(
            log_messages[0],
            ("INFO elifecleaner:prc:license_from_docmap: " "Parse docmap json\n"),
        )

    def test_no_license(self):
        "test if no license data can be found in the docmap"
        docmap_json = docmap_test_data()
        del docmap_json["steps"]["_:b1"]["actions"][0]["outputs"][0]["license"]
        expected = None
        # invoke
        result = prc.license_from_docmap(
            json.dumps(docmap_json), identifier=self.identifier
        )
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
        # assert
        self.assertEqual(result, expected)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[-1],
                (
                    "WARNING elifecleaner:prc:license_from_docmap: "
                    "%s no license found in the docmap\n"
                )
                % self.identifier,
            )

    def test_docmap_is_none(self):
        "test if docmap is empty"
        docmap_json = {}
        # invoke
        result = prc.license_from_docmap(
            json.dumps(docmap_json), identifier=self.identifier
        )
        # assert
        self.assertEqual(result, None)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[-1],
                (
                    "WARNING elifecleaner:prc:license_from_docmap: "
                    "%s parsing docmap returned None\n"
                )
                % self.identifier,
            )


class TestVolumeFromDocmap(unittest.TestCase):
    "tests for prc.volume_from_docmap()"

    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)
        self.identifier = "test.zip"

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_volume_from_docmap(self):
        "parse volume from docmap data"
        docmap_json = docmap_test_data()
        expected = 12
        # invoke
        volume = prc.volume_from_docmap(
            json.dumps(docmap_json), identifier=self.identifier
        )
        # assert
        self.assertEqual(volume, expected)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
        self.assertEqual(
            log_messages[0],
            ("INFO elifecleaner:prc:volume_from_docmap: " "Parse docmap json\n"),
        )

    def test_volume_non_int(self):
        "test if the volume value cannot be parsed to int"
        docmap_json = docmap_test_data()
        docmap_json["steps"]["_:b1"]["actions"][0]["outputs"][0]["partOf"][
            "volumeIdentifier"
        ] = "foo"
        expected = None
        # invoke
        volume = prc.volume_from_docmap(
            json.dumps(docmap_json), identifier=self.identifier
        )
        # assert
        self.assertEqual(volume, expected)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
        self.assertEqual(
            log_messages[0],
            ("INFO elifecleaner:prc:volume_from_docmap: " "Parse docmap json\n"),
        )
        self.assertEqual(
            log_messages[-1],
            (
                "WARNING elifecleaner:prc:volume_from_docmap: "
                "test.zip volume from the docmap could not be cast as int\n"
            ),
        )

    def test_no_history_data(self):
        "test if no history data can be found in the docmap"
        docmap_json = {"first-step": "_:b0", "steps": {"_:b0": {}}}
        expected = None
        # invoke
        volume = prc.volume_from_docmap(
            json.dumps(docmap_json), identifier=self.identifier
        )
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
        # assert
        self.assertEqual(volume, expected)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[-1],
                (
                    "WARNING elifecleaner:prc:volume_from_docmap: "
                    "%s no volume found in the docmap\n"
                )
                % self.identifier,
            )

    def test_docmap_is_none(self):
        "test if docmap is empty"
        docmap_json = {}
        # invoke
        volume = prc.volume_from_docmap(
            json.dumps(docmap_json), identifier=self.identifier
        )
        # assert
        self.assertEqual(volume, None)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[-1],
                (
                    "WARNING elifecleaner:prc:volume_from_docmap: "
                    "%s parsing docmap returned None\n"
                )
                % self.identifier,
            )


class TestArticleCategoriesFromDocmap(unittest.TestCase):
    "tests for article_categories_from_docmap()"

    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)
        self.identifier = "test.zip"

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_article_categories(self):
        "parse article categories from docmap preprint output"
        docmap_json = docmap_test_data()
        expected = ["Neuroscience"]
        # invoke
        result = prc.article_categories_from_docmap(
            json.dumps(docmap_json), identifier=self.identifier
        )
        # assert
        self.assertEqual(result, expected)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
        self.assertEqual(
            log_messages[0],
            (
                "INFO elifecleaner:prc:article_categories_from_docmap: "
                "Parse docmap json\n"
            ),
        )

    def test_no_article_categories(self):
        "test if no article categories data can be found in the docmap"
        docmap_json = docmap_test_data()
        del docmap_json["steps"]["_:b1"]["actions"][0]["outputs"][0]["partOf"]
        expected = None
        # invoke
        result = prc.article_categories_from_docmap(
            json.dumps(docmap_json), identifier=self.identifier
        )
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
        # assert
        self.assertEqual(result, expected)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[-1],
                (
                    "WARNING elifecleaner:prc:article_categories_from_docmap: "
                    "%s no article_categories found in the docmap\n"
                )
                % self.identifier,
            )

    def test_docmap_is_none(self):
        "test if docmap is empty"
        docmap_json = {}
        # invoke
        result = prc.article_categories_from_docmap(
            json.dumps(docmap_json), identifier=self.identifier
        )
        # assert
        self.assertEqual(result, None)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[-1],
                (
                    "WARNING elifecleaner:prc:article_categories_from_docmap: "
                    "%s parsing docmap returned None\n"
                )
                % self.identifier,
            )


class TestDateStructFromString(unittest.TestCase):
    "tests for prc.date_struct_from_string()"

    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)
        self.identifier = "test.zip"

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_with_timezone(self):
        "test docmap which has a review date"
        date_string = "2022-11-28T11:30:05+00:00"
        expected = time.strptime(date_string, "%Y-%m-%dT%H:%M:%S%z")
        result = prc.date_struct_from_string(date_string)
        self.assertEqual(result, expected)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[-1],
                (
                    "INFO elifecleaner:prc:date_struct_from_string: "
                    'unable to parse "%s" using format "%s"\n'
                )
                % (date_string, "%Y-%m-%dT%H:%M:%S.%f%z"),
            )

    def test_date(self):
        "test docmap which has a review date"
        date_string = "2022-11-28"
        expected = time.strptime(date_string, "%Y-%m-%d")
        result = prc.date_struct_from_string(date_string)
        self.assertEqual(result, expected)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[-1],
                (
                    "INFO elifecleaner:prc:date_struct_from_string: "
                    'unable to parse "%s" using format "%s"\n'
                )
                % (date_string, "%Y-%m-%dT%H:%M:%S%z"),
            )

    def test_with_microtime(self):
        "test docmap which has a review date"
        date_string = "2022-11-28T11:30:05.579531+00:00"
        expected = time.strptime(date_string, "%Y-%m-%dT%H:%M:%S.%f%z")
        result = prc.date_struct_from_string(date_string)
        self.assertEqual(result, expected)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(log_messages, [])

    def test_not_a_date(self):
        "test docmap which has a review date"
        date_string = "not_a_date"
        expected = None
        result = prc.date_struct_from_string(date_string)
        self.assertEqual(result, expected)
        with open(self.log_file, "r") as open_file:
            log_messages = open_file.readlines()
            self.assertEqual(
                log_messages[0],
                (
                    "INFO elifecleaner:prc:date_struct_from_string: "
                    'unable to parse "%s" using format "%s"\n'
                )
                % (date_string, "%Y-%m-%dT%H:%M:%S.%f%z"),
            )


class TestAddHistoryDate(unittest.TestCase):
    "tests for prc.add_history_date()"

    def setUp(self):
        self.xml_string_template = (
            "<article><front><article-meta>%s</article-meta></front></article>"
        )
        self.date_type = "sent-for-review"
        date_string = "2022-11-28"
        self.date_struct = time.strptime(date_string, "%Y-%m-%d")
        self.identifier = "test.zip"
        # expected history XML string for when using the input values
        self.history_xml_output = (
            "<history>"
            '<date date-type="sent-for-review" iso-8601-date="2022-11-28">'
            "<day>28</day>"
            "<month>11</month>"
            "<year>2022</year>"
            "</date>"
            "</history>"
        )

    def test_add_history_date(self):
        "test adding a date to an existing history tag"
        xml_string = self.xml_string_template % "<history />"
        expected = bytes(
            self.xml_string_template % self.history_xml_output, encoding="utf-8"
        )
        root = ElementTree.fromstring(xml_string)
        root_output = prc.add_history_date(
            root, self.date_type, self.date_struct, self.identifier
        )
        self.assertEqual(ElementTree.tostring(root_output), expected)

    def test_no_history_tag(self):
        "test if there is no history tag"
        xml_string = self.xml_string_template % ""
        expected = bytes(
            self.xml_string_template % self.history_xml_output, encoding="utf-8"
        )
        root = ElementTree.fromstring(xml_string)
        root_output = prc.add_history_date(
            root, self.date_type, self.date_struct, self.identifier
        )
        self.assertEqual(ElementTree.tostring(root_output), expected)

    def test_elocation_id_tag(self):
        "test history tag should be added after the elocation-id tag"
        xml_string = self.xml_string_template % "<elocation-id />"
        expected = bytes(
            self.xml_string_template % ("<elocation-id />" + self.history_xml_output),
            encoding="utf-8",
        )
        root = ElementTree.fromstring(xml_string)
        root_output = prc.add_history_date(
            root, self.date_type, self.date_struct, self.identifier
        )
        self.assertEqual(ElementTree.tostring(root_output), expected)

    def test_no_article_meta(self):
        "test if there is no article-meta tag"
        xml_string = "<article />"
        expected = b"<article />"
        root = ElementTree.fromstring(xml_string)
        root_output = prc.add_history_date(
            root, self.date_type, self.date_struct, self.identifier
        )
        self.assertEqual(ElementTree.tostring(root_output), expected)


class TestSetArticleId(unittest.TestCase):
    "tests for set_article_id()"

    def test_set_article_id(self):
        "test adding various article-id tags"
        xml_string = "<article><front><article-meta /></front></article>"
        xml_root = ElementTree.fromstring(xml_string)
        article_id = "95901"
        doi = "10.7554/eLife.95901"
        version_doi = "10.7554/eLife.95901.1"
        expected = (
            "<article>"
            "<front>"
            "<article-meta>"
            '<article-id pub-id-type="publisher-id">%s</article-id>'
            '<article-id pub-id-type="doi">%s</article-id>'
            '<article-id pub-id-type="doi" specific-use="version">%s</article-id>'
            "</article-meta>"
            "</front>"
            "</article>" % (article_id, doi, version_doi)
        )
        # invoke
        prc.set_article_id(xml_root, article_id, doi, version_doi)
        # assert
        self.assertEqual(ElementTree.tostring(xml_root).decode("utf-8"), expected)


class TestSetVolume(unittest.TestCase):
    "tests for set_volume()"

    def test_modify(self):
        "test modifying existing volume tag"
        xml_string = (
            "<article><front><article-meta>"
            "<volume>foo</volume>"
            "</article-meta></front></article>"
        )
        root = ElementTree.fromstring(xml_string)
        volume = "12"
        expected = (
            "<article><front><article-meta>"
            "<volume>12</volume>"
            "</article-meta></front></article>"
        )
        prc.set_volume(root, volume)
        self.assertEqual(ElementTree.tostring(root).decode("utf8"), expected)

    def test_insert_before(self):
        "test inserting volume tag before elocation-id tag"
        xml_string = (
            "<article><front><article-meta>"
            "<elocation-id />"
            "</article-meta></front></article>"
        )
        root = ElementTree.fromstring(xml_string)
        volume = "12"
        expected = (
            "<article><front><article-meta>"
            "<volume>12</volume>"
            "<elocation-id />"
            "</article-meta></front></article>"
        )
        prc.set_volume(root, volume)
        self.assertEqual(ElementTree.tostring(root).decode("utf8"), expected)

    def test_append(self):
        "test appending volume tag if no elocation-id tag"
        xml_string = "<article><front><article-meta /></front></article>"
        root = ElementTree.fromstring(xml_string)
        volume = "12"
        expected = (
            "<article><front><article-meta>"
            "<volume>12</volume>"
            "</article-meta></front></article>"
        )
        prc.set_volume(root, volume)
        self.assertEqual(ElementTree.tostring(root).decode("utf8"), expected)


class TestSetElocationId(unittest.TestCase):
    "tests for set_elocation_id()"

    def test_set_elocation_id(self):
        "test modifying existing elocation-id tag"
        xml_string = (
            "<article><front><article-meta>"
            "<elocation-id>foo</elocation-id>"
            "</article-meta></front></article>"
        )
        root = ElementTree.fromstring(xml_string)
        elocation_id = "RP95901"
        expected = (
            "<article><front><article-meta>"
            "<elocation-id>RP95901</elocation-id>"
            "</article-meta></front></article>"
        )
        prc.set_elocation_id(root, elocation_id)
        self.assertEqual(ElementTree.tostring(root).decode("utf8"), expected)

    def test_insert_after(self):
        "test inserting elocation-id tag after volume tag"
        xml_string = (
            "<article><front><article-meta>"
            "<volume />"
            "</article-meta></front></article>"
        )
        root = ElementTree.fromstring(xml_string)
        elocation_id = "RP95901"
        expected = (
            "<article><front><article-meta>"
            "<volume />"
            "<elocation-id>RP95901</elocation-id>"
            "</article-meta></front></article>"
        )
        prc.set_elocation_id(root, elocation_id)
        self.assertEqual(ElementTree.tostring(root).decode("utf8"), expected)

    def test_append(self):
        "test appending volume tag if no elocation-id tag"
        xml_string = "<article><front><article-meta /></front></article>"
        root = ElementTree.fromstring(xml_string)
        elocation_id = "RP95901"
        expected = (
            "<article><front><article-meta>"
            "<elocation-id>RP95901</elocation-id>"
            "</article-meta></front></article>"
        )
        prc.set_elocation_id(root, elocation_id)
        self.assertEqual(ElementTree.tostring(root).decode("utf8"), expected)


class TestSetArticleCategories(unittest.TestCase):
    "tests for set_article_categories()"

    def test_set_article_categories(self):
        "test adding subject tags to article-categories"
        xml_root = ElementTree.fromstring(
            "<article>"
            "<front>"
            "<article-meta>"
            "<article-categories />"
            "</article-meta>"
            "</front>"
            "</article>"
        )
        display_channel = "Research Article"
        article_categories = ["Structural Biology and Molecular Biophysics"]
        expected = (
            "<article>"
            "<front>"
            "<article-meta>"
            "<article-categories>"
            '<subj-group subj-group-type="display-channel">'
            "<subject>Research Article</subject>"
            "</subj-group>"
            '<subj-group subj-group-type="heading">'
            "<subject>Structural Biology and Molecular Biophysics</subject>"
            "</subj-group>"
            "</article-categories>"
            "</article-meta>"
            "</front>"
            "</article>"
        )
        # invoke
        prc.set_article_categories(xml_root, display_channel, article_categories)
        # assert
        xml_string = ElementTree.tostring(xml_root).decode("utf-8")
        self.assertEqual(xml_string, expected)

    def test_no_article_categories_tag(self):
        "test inserting article-categories tag"
        xml_root = ElementTree.fromstring(
            "<article>"
            "<front>"
            "<article-meta>"
            "<article-id />"
            "<title-group />"
            "</article-meta>"
            "</front>"
            "</article>"
        )
        display_channel = "Research Article"
        article_categories = ["Structural Biology and Molecular Biophysics"]
        expected = (
            "<article>"
            "<front>"
            "<article-meta>"
            "<article-id />"
            "<article-categories>"
            '<subj-group subj-group-type="display-channel">'
            "<subject>Research Article</subject>"
            "</subj-group>"
            '<subj-group subj-group-type="heading">'
            "<subject>Structural Biology and Molecular Biophysics</subject>"
            "</subj-group>"
            "</article-categories>"
            "<title-group />"
            "</article-meta>"
            "</front>"
            "</article>"
        )
        # invoke
        prc.set_article_categories(xml_root, display_channel, article_categories)
        # assert
        xml_string = ElementTree.tostring(xml_root).decode("utf-8")
        self.assertEqual(xml_string, expected)

    def test_alternatives_tag(self):
        "test if there is also an article-version-alternatives tag"
        xml_root = ElementTree.fromstring(
            "<article>"
            "<front>"
            "<article-meta>"
            "<article-id />"
            "<article-version-alternatives/>"
            "<title-group />"
            "</article-meta>"
            "</front>"
            "</article>"
        )
        display_channel = "Research Article"
        article_categories = ["Structural Biology and Molecular Biophysics"]
        expected = (
            "<article>"
            "<front>"
            "<article-meta>"
            "<article-id />"
            "<article-version-alternatives />"
            "<article-categories>"
            '<subj-group subj-group-type="display-channel">'
            "<subject>Research Article</subject>"
            "</subj-group>"
            '<subj-group subj-group-type="heading">'
            "<subject>Structural Biology and Molecular Biophysics</subject>"
            "</subj-group>"
            "</article-categories>"
            "<title-group />"
            "</article-meta>"
            "</front>"
            "</article>"
        )
        # invoke
        prc.set_article_categories(xml_root, display_channel, article_categories)
        # assert
        xml_string = ElementTree.tostring(xml_root).decode("utf-8")
        self.assertEqual(xml_string, expected)


# cc-by license data
LICENSE_DATA_DICT_CC_BY = {
    "license_id": 1,
    "license_type": "open-access",
    "copyright": True,
    "href": "http://creativecommons.org/licenses/by/4.0/",
    "name": "Creative Commons Attribution License",
    "paragraph1": "This article is distributed under the terms of the ",
    "paragraph2": (
        ", which permits unrestricted use and redistribution provided that the"
        " original author and source are credited."
    ),
}

# cc-0 license data
LICENSE_DATA_DICT_CC_0 = {
    "license_id": 2,
    "license_type": "open-access",
    "copyright": False,
    "href": "http://creativecommons.org/publicdomain/zero/1.0/",
    "name": "Creative Commons CC0 public domain dedication",
    "paragraph1": (
        "This is an open-access article, free of all copyright, and may be"
        " freely reproduced, distributed, transmitted, modified, built upon, or"
        " otherwise used by anyone for any lawful purpose. The work is made"
        " available under the "
    ),
    "paragraph2": ".",
}


class TestLicenseTag(unittest.TestCase):
    "tests for license_tag()"

    def setUp(self):
        # register XML namespaces
        xmlio.register_xmlns()

    def test_cc_by(self):
        "test setting CC-BY license tag"
        parent = ElementTree.fromstring("<permissions />")
        expected = (
            '<permissions xmlns:ali="http://www.niso.org/schemas/ali/1.0/"'
            ' xmlns:xlink="http://www.w3.org/1999/xlink">'
            '<license xlink:href="http://creativecommons.org/licenses/by/4.0/">'
            "<ali:license_ref>"
            "http://creativecommons.org/licenses/by/4.0/"
            "</ali:license_ref>"
            "<license-p>"
            "This article is distributed under the terms of the"
            ' <ext-link ext-link-type="uri"'
            ' xlink:href="http://creativecommons.org/licenses/by/4.0/">'
            "Creative Commons Attribution License"
            "</ext-link>"
            ", which permits unrestricted use and redistribution provided that the"
            " original author and source are credited."
            "</license-p>"
            "</license>"
            "</permissions>"
        )
        # invoke
        prc.set_license_tag(parent, LICENSE_DATA_DICT_CC_BY)
        # assert
        xml_string = ElementTree.tostring(parent).decode("utf-8")
        self.assertEqual(xml_string, expected)

    def test_cc_0(self):
        "test setting CC-0 license tag"
        parent = ElementTree.fromstring("<permissions />")
        expected = (
            '<permissions xmlns:ali="http://www.niso.org/schemas/ali/1.0/"'
            ' xmlns:xlink="http://www.w3.org/1999/xlink">'
            '<license xlink:href="http://creativecommons.org/publicdomain/zero/1.0/">'
            "<ali:license_ref>"
            "http://creativecommons.org/publicdomain/zero/1.0/"
            "</ali:license_ref>"
            "<license-p>"
            "This is an open-access article, free of all copyright, and may be freely"
            " reproduced, distributed, transmitted, modified, built upon, or otherwise"
            " used by anyone for any lawful purpose. The work is made available under the"
            ' <ext-link ext-link-type="uri"'
            ' xlink:href="http://creativecommons.org/publicdomain/zero/1.0/">'
            "Creative Commons CC0 public domain dedication"
            "</ext-link>."
            "</license-p>"
            "</license>"
            "</permissions>"
        )
        # invoke
        prc.set_license_tag(parent, LICENSE_DATA_DICT_CC_0)
        # assert
        xml_string = ElementTree.tostring(parent).decode("utf-8")
        self.assertEqual(xml_string, expected)

    def test_no_data(self):
        "test if no data is provided"
        parent = ElementTree.fromstring("<permissions />")
        license_data_dict = {}
        expected = "<permissions />"
        # invoke
        prc.set_license_tag(parent, license_data_dict)
        # assert
        xml_string = ElementTree.tostring(parent).decode("utf-8")
        self.assertEqual(xml_string, expected)


class TestSetPermissions(unittest.TestCase):
    "tests for set_permissions()"

    def test_set_permissions(self):
        "test setting permissions and license XML"
        xml_root = ElementTree.fromstring(
            "<article>"
            "<front>"
            "<article-meta>"
            "<history />"
            "<pub-history />"
            "</article-meta>"
            "</front>"
            "</article>"
        )
        copyright_year = "2024"
        copyright_holder = "Surname et al"
        expected = (
            '<article xmlns:ali="http://www.niso.org/schemas/ali/1.0/"'
            ' xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<front>"
            "<article-meta>"
            "<history />"
            "<pub-history />"
            "<permissions>"
            "<copyright-statement>© 2024, Surname et al</copyright-statement>"
            "<copyright-year>2024</copyright-year>"
            "<copyright-holder>Surname et al</copyright-holder>"
            "<ali:free_to_read />"
            '<license xlink:href="http://creativecommons.org/licenses/by/4.0/">'
            "<ali:license_ref>http://creativecommons.org/licenses/by/4.0/</ali:license_ref>"
            "<license-p>This article is distributed under the terms of the"
            ' <ext-link ext-link-type="uri"'
            ' xlink:href="http://creativecommons.org/licenses/by/4.0/">'
            "Creative Commons Attribution License"
            "</ext-link>, which permits unrestricted use and redistribution provided that"
            " the original author and source are credited."
            "</license-p>"
            "</license>"
            "</permissions>"
            "</article-meta>"
            "</front>"
            "</article>"
        )
        # invoke
        prc.set_permissions(
            xml_root, LICENSE_DATA_DICT_CC_BY, copyright_year, copyright_holder
        )
        # assert
        xml_string = ElementTree.tostring(xml_root, encoding="utf-8").decode("utf-8")
        self.assertEqual(xml_string, expected)


class TestEditorContributors(unittest.TestCase):
    "tests for editor_contributors()"

    def test_editor_contributors(self):
        "test populating editor Contributor objects"
        docmap_string = read_fixture("99854.json", mode="r")
        version_doi = "10.7554/eLife.99854.1"
        # invoke
        editors = prc.editor_contributors(docmap_string, version_doi)
        # assert
        self.assertEqual(len(editors), 2)
        self.assertEqual(editors[0].surname, "Proenza")
        self.assertEqual(editors[0].contrib_type, "editor")
        self.assertEqual(
            str(editors[0].affiliations[0]),
            (
                "{'institution': 'University of Colorado Anschutz Medical Campus',"
                " 'city': 'Aurora', 'country': 'United States of America'}"
            ),
        )
        self.assertEqual(editors[1].surname, "Swartz")
        self.assertEqual(editors[1].contrib_type, "senior_editor")
        self.assertEqual(
            str(editors[1].affiliations[0]),
            (
                "{'institution': 'National Institute of Neurological Disorders and Stroke',"
                " 'city': 'Bethesda', 'country': 'United States of America'}"
            ),
        )


class TestSetEditors(unittest.TestCase):
    "tests for set_editors()"

    def setUp(self):
        self.editor = Contributor(
            contrib_type="editor", surname="Surname", given_name="Given"
        )
        editor_role = Role()
        editor_role.text = "Reviewing Editor"
        self.editor.roles = [editor_role]
        aff = Affiliation()
        aff.city = "City"
        aff.country = "Country"
        self.editor.set_affiliation(aff)

    def test_set_editors(self):
        "test setting editor contrib tags"
        parent = ElementTree.fromstring("<root><contrib-group /></root>")
        editors = [self.editor]
        expected = (
            "<root>"
            "<contrib-group />"
            '<contrib-group content-type="section">'
            '<contrib contrib-type="editor">'
            "<name>"
            "<surname>Surname</surname>"
            "<given-names>Given</given-names>"
            "</name>"
            "<role>Reviewing Editor</role>"
            "<aff>"
            "<addr-line>"
            '<named-content content-type="city">City</named-content>'
            "</addr-line>"
            "<country>Country</country>"
            "</aff>"
            "</contrib>"
            "</contrib-group>"
            "</root>"
        )
        # invoke
        prc.set_editors(parent, editors)
        # assert
        xml_string = ElementTree.tostring(parent).decode("utf-8")
        self.assertEqual(xml_string, expected)

    def test_append(self):
        "test appending tag when insert order is not determined"
        parent = ElementTree.fromstring("<root></root>")
        editors = [self.editor]
        # invoke
        prc.set_editors(parent, editors)
        # assert
        xml_string = ElementTree.tostring(parent).decode("utf-8")
        self.assertTrue('<contrib-group content-type="section">' in xml_string)

    def test_no_editors(self):
        "test if there is no editor data"
        parent = ElementTree.fromstring("<root />")
        editors = []
        expected = "<root />"
        # invoke
        prc.set_editors(parent, editors)
        # assert
        xml_string = ElementTree.tostring(parent).decode("utf-8")
        self.assertEqual(xml_string, expected)



import os
import unittest
from collections import OrderedDict
from elifecleaner import configure_logging, video
from tests.helpers import delete_files_in_folder, read_log_file_lines


VIDEO_TITLE_EXAMPLES = [
    {
        "comment": "Example of Animation",
        "article_id": 63107,
        "title": "Animation 1",
        "upload_file_nm": "Animation 1.gif",
        "id": "video1",
        "filename": "elife-63107-video1.gif",
    },
    {
        "comment": "Example of Appendix",
        "article_id": 64000,
        "title": "Appendix 12-figure 1-video 1",
        "upload_file_nm": "Appendix 12 Figure A1video 1.mov",
        "id": "app12fig1video1",
        "filename": "elife-64000-app12-fig1-video1.mov",
    },
    {
        "comment": "Example including Audio will be ignored",
        "article_id": 62329,
        "title": "Audio 1",
        "upload_file_nm": "MaleNarrator_unaltered.mp4",
        "id": None,
        "filename": None,
    },
    {
        "comment": "Example of figure animation",
        "article_id": 56603,
        "title": "Figure 2-animation 1",
        "upload_file_nm": "violindecoding_movie.gif",
        "id": "fig2video1",
        "filename": "elife-56603-fig2-video1.gif",
    },
    {
        "comment": "Example of video supplement",
        "article_id": 41328,
        "title": "Figure 1-video supplement 1",
        "upload_file_nm": "task low noise.avi",
        "id": "fig1video1",
        "filename": "elife-41328-fig1-video1.avi",
    },
    {
        "comment": "Example of videos supplement, plural",
        "article_id": 52986,
        "title": "Figure 1-Videos Supplement 1",
        "upload_file_nm": "Figure 1 Video Supplemental 1A.mp4",
        "id": "fig1video1",
        "filename": "elife-52986-fig1-video1.mp4",
    },
    {
        "comment": "Example of Figure Supplement",
        "article_id": 58962,
        "title": "Figure 1 - Figure Supplement 3 - video 2",
        "upload_file_nm": (
            "Figure 1 Figure supplement 3 video 2. wildtype_microdomain transients in L1 CNS.avi"
        ),
        "id": "fig1video2",
        "filename": "elife-58962-fig1-video2.avi",
    },
    {
        "comment": "Example of Supplementary Video",
        "article_id": 43542,
        "title": "Figure 9 - Supplementary Video 1",
        "upload_file_nm": "supp_figure9_video1.mp4",
        "id": "fig9video1",
        "filename": "elife-43542-fig9-video1.mp4",
    },
    {
        "comment": "Example of extra whitespace",
        "article_id": 51221,
        "title": "Video  1",
        "upload_file_nm": "video 1.mp4",
        "id": "video1",
        "filename": "elife-51221-video1.mp4",
    },
    {
        "comment": "Example of no whitespace",
        "article_id": 58145,
        "title": "Figure2-Video1",
        "upload_file_nm": "Figure2Movie1.mp4",
        "id": "fig2video1",
        "filename": "elife-58145-fig2-video1.mp4",
    },
    {
        "comment": "Example with underscore",
        "article_id": 62047,
        "title": "Video_10",
        "upload_file_nm": "Video_10.mp4",
        "id": "video10",
        "filename": "elife-62047-video10.mp4",
    },
    {
        "comment": "Example with underscore and two elements",
        "article_id": 63755,
        "title": "Figure 7_video 1",
        "upload_file_nm": "Figure 7Video 1_Custodio et al_time lapse of colony in Fig.7A.avi",
        "id": "fig7video1",
        "filename": "elife-63755-fig7-video1.avi",
    },
    {
        "comment": "Example with a file extension",
        "article_id": 58626,
        "title": "Figure 1 Video 1.mp4",
        "upload_file_nm": "Figure 1_Video 1.mp4",
        "id": "fig1video1",
        "filename": "elife-58626-fig1-video1.mp4",
    },
    {
        "comment": "Example of a longer title",
        "article_id": 40033,
        "title": "Figure 7-video 2. Murine Fgf2 +/+ hematopoietic progenitors and ....",
        "upload_file_nm": "mouse DiI KO exosomes  DiO WT BM lin neg.mov",
        "id": "fig7video2",
        "filename": "elife-40033-fig7-video2.mov",
    },
    {
        "comment": "Example of missing number",
        "article_id": 99999,
        "title": "Figure - Video 1",
        "upload_file_nm": "video.mp4",
        "id": None,
        "filename": None,
    },
    {
        "comment": "Example of a non-numeric figure number",
        "article_id": 65234,
        "title": "Figure 6A-video 1",
        "upload_file_nm": "Figure 6A Video 1.mp4",
        "id": None,
        "filename": None,
    },
    {
        "comment": "Example of missing video term",
        "article_id": 42823,
        "title": "Figure 3-figure supplement 2",
        "upload_file_nm": (
            "Figure 3figure supplement 2. "
            "Six Chromosome FISH of a C. elegans intestinal nucleus.mp4"
        ),
        "id": None,
        "filename": None,
    },
    {
        "comment": "Example of video typo",
        "article_id": 47346,
        "title": "Vidoe 5",
        "upload_file_nm": "Video 5.mp4",
        "id": None,
        "filename": None,
    },
    {
        "comment": "Example of irregular title value",
        "article_id": 53777,
        "title": "Potential Striking Image- Live and dynamic IP",
        "upload_file_nm": "INP_DNeDMS_004 copy.mp4",
        "id": None,
        "filename": None,
    },
]

VIDEO_FILES_EXAMPLE = [
    OrderedDict(
        [
            ("file_type", "video"),
            ("upload_file_nm", "Video 1 AVI.avi"),
            (
                "custom_meta",
                [
                    OrderedDict(
                        [
                            ("meta_name", "Title"),
                            ("meta_value", "Video 1"),
                        ]
                    )
                ],
            ),
        ]
    ),
    OrderedDict(
        [
            ("file_type", "video"),
            ("upload_file_nm", "Video 2 AVI.avi"),
            (
                "custom_meta",
                [
                    OrderedDict(
                        [
                            ("meta_name", "Title"),
                            ("meta_value", "Audio 1"),
                        ]
                    )
                ],
            ),
        ]
    ),
    OrderedDict(
        [
            ("file_type", "figure"),
            ("upload_file_nm", "eLife64719_figure1_classB.png"),
            (
                "custom_meta",
                [
                    OrderedDict(
                        [
                            ("meta_name", "Figure number"),
                            ("meta_value", "Figure 1"),
                        ]
                    ),
                    OrderedDict(
                        [
                            ("meta_name", "Title"),
                            ("meta_value", "Figure 1"),
                        ]
                    ),
                ],
            ),
        ]
    ),
]


class TestVideoFileList(unittest.TestCase):
    def test_video_file_list(self):
        files = VIDEO_FILES_EXAMPLE
        expected = VIDEO_FILES_EXAMPLE[0:2]
        self.assertEqual(video.video_file_list(files), expected)


class TestCollectVideoData(unittest.TestCase):
    def test_collect_video_data(self):
        files = VIDEO_FILES_EXAMPLE
        expected = [
            OrderedDict([("upload_file_nm", "Video 1 AVI.avi"), ("title", "Video 1")]),
            OrderedDict([("upload_file_nm", "Video 2 AVI.avi"), ("title", "Audio 1")]),
        ]
        self.assertEqual(video.collect_video_data(files), expected)


class TestRenameVideoData(unittest.TestCase):
    def test_rename_video_data(self):
        article_id = "3"
        video_data = [{"upload_file_nm": "Video 1.ogv", "title": "Figure 1 - Video 1"}]
        expected = [
            OrderedDict(
                [
                    ("upload_file_nm", "Video 1.ogv"),
                    ("video_id", "fig1video1"),
                    ("video_filename", "elife-00003-fig1-video1.ogv"),
                ]
            )
        ]
        self.assertEqual(video.rename_video_data(video_data, article_id), expected)


class TestVideoDataFromFiles(unittest.TestCase):
    def test_video_data_from_files(self):
        article_id = "3"
        files = VIDEO_FILES_EXAMPLE
        expected = [
            OrderedDict(
                [
                    ("upload_file_nm", "Video 1 AVI.avi"),
                    ("video_id", "video1"),
                    ("video_filename", "elife-00003-video1.avi"),
                ]
            ),
            OrderedDict(
                [
                    ("upload_file_nm", "Video 2 AVI.avi"),
                    ("video_id", None),
                    ("video_filename", None),
                ]
            ),
        ]
        self.assertEqual(video.video_data_from_files(files, article_id), expected)

    def test_video_data_from_files_empty(self):
        files = [""]
        article_id = "3"
        expected = []
        self.assertEqual(video.video_data_from_files(files, article_id), expected)


class TestAllTermsMap(unittest.TestCase):
    def test_all_terms_map(self):
        video_data = [
            OrderedDict(
                [
                    ("upload_file_nm", "Video 1.ogv"),
                    ("title", "Figure 1 - Video 1"),
                ]
            ),
            OrderedDict(
                [
                    ("upload_file_nm", "Video 2.mp4"),
                    ("title", "Figure 1 - Video 2"),
                ]
            ),
            OrderedDict(
                [
                    ("upload_file_nm", "Appendix 2 Video 1.mp4"),
                    ("title", "Appendix 2 Video 1"),
                ]
            ),
        ]

        expected = OrderedDict(
            [
                (
                    "Video 1.ogv",
                    [
                        OrderedDict([("name", "fig"), ("number", "1")]),
                        OrderedDict([("name", "video"), ("number", "1")]),
                    ],
                ),
                (
                    "Video 2.mp4",
                    [
                        OrderedDict([("name", "fig"), ("number", "1")]),
                        OrderedDict([("name", "video"), ("number", "2")]),
                    ],
                ),
                (
                    "Appendix 2 Video 1.mp4",
                    [
                        OrderedDict([("name", "app"), ("number", "2")]),
                        OrderedDict([("name", "video"), ("number", "1")]),
                    ],
                ),
            ]
        )
        all_terms = video.all_terms_map(video_data)
        self.assertEqual(all_terms, expected, "got all_terms: %s" % all_terms)

    def test_all_terms_map_duplicate(self):
        "check return value when there is a duplicate video file name"
        video_data = [
            OrderedDict(
                [
                    ("upload_file_nm", "Video 2.ogv"),
                    ("title", "Figure 1 - Video 2"),
                ]
            ),
            OrderedDict(
                [
                    ("upload_file_nm", "Supplementary Video 2.mp4"),
                    ("title", "Figure 1 - Supplementary Video 2"),
                ]
            ),
            OrderedDict(
                [
                    ("upload_file_nm", "Video 1.mp4"),
                    ("title", "Figure 1 - Video 1"),
                ]
            ),
        ]
        # note the term values are now unique, the duplicate value is renumbered
        expected = OrderedDict(
            [
                (
                    "Video 2.ogv",
                    [
                        OrderedDict([("name", "fig"), ("number", "1")]),
                        OrderedDict([("name", "video"), ("number", "2")]),
                    ],
                ),
                (
                    "Supplementary Video 2.mp4",
                    [
                        OrderedDict([("name", "fig"), ("number", "1")]),
                        OrderedDict([("name", "video"), ("number", "3")]),
                    ],
                ),
                (
                    "Video 1.mp4",
                    [
                        OrderedDict([("name", "fig"), ("number", "1")]),
                        OrderedDict([("name", "video"), ("number", "1")]),
                    ],
                ),
            ]
        )
        all_terms = video.all_terms_map(video_data)
        self.assertEqual(all_terms, expected, "got all_terms: %s" % all_terms)


class TestRenumberTermMap(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def tearDown(self):
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_renumber_term_map(self):
        configure_logging(self.log_file)
        term_map = OrderedDict(
            [
                (
                    "Video 2.ogv",
                    [
                        OrderedDict([("name", "fig"), ("number", "1")]),
                        OrderedDict([("name", "video"), ("number", "2")]),
                    ],
                ),
                (
                    "Supplementary Video 2.mp4",
                    [
                        OrderedDict([("name", "fig"), ("number", "1")]),
                        OrderedDict([("name", "video"), ("number", "2")]),
                    ],
                ),
                (
                    "Video 1.mp4",
                    [
                        OrderedDict([("name", "fig"), ("number", "1")]),
                        OrderedDict([("name", "video"), ("number", "1")]),
                    ],
                ),
            ]
        )
        prefix_to_keys_map = {
            "fig1video": ["Video 2.ogv", "Supplementary Video 2.mp4", "Video 1.mp4"]
        }

        expected = OrderedDict(
            [
                (
                    "Video 2.ogv",
                    [
                        OrderedDict([("name", "fig"), ("number", "1")]),
                        OrderedDict([("name", "video"), ("number", "2")]),
                    ],
                ),
                (
                    "Supplementary Video 2.mp4",
                    [
                        OrderedDict([("name", "fig"), ("number", "1")]),
                        OrderedDict([("name", "video"), ("number", "3")]),
                    ],
                ),
                (
                    "Video 1.mp4",
                    [
                        OrderedDict([("name", "fig"), ("number", "1")]),
                        OrderedDict([("name", "video"), ("number", "1")]),
                    ],
                ),
            ]
        )
        info_prefix = "INFO elifecleaner:video:renumber_term_map:"
        expected_log_file_line = (
            "%s replacing number 2 with 3 for term Supplementary Video 2.mp4\n"
            % info_prefix
        )

        new_term_map = video.renumber_term_map(term_map, prefix_to_keys_map)
        self.assertEqual(new_term_map, expected)
        self.assertEqual(read_log_file_lines(self.log_file)[-1], expected_log_file_line)


class TestRenumberKeyMap(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def tearDown(self):
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_renumber_key_map(self):
        configure_logging(self.log_file)
        prefix = "fig1video"
        key_map = OrderedDict(
            [
                ("Video 2.ogv", 2),
                ("Supplementary Video 2.mp4", 2),
                ("Video 1.mp4", 1),
            ]
        )

        expected = OrderedDict(
            [
                ("Video 2.ogv", 2),
                ("Supplementary Video 2.mp4", 3),
                ("Video 1.mp4", 1),
            ]
        )
        info_prefix = "INFO elifecleaner:video:renumber_key_map:"
        expected_log_file_lines = [
            "%s number values used for video prefix fig1video: [2, 2, 1]\n"
            % info_prefix,
            "%s duplicate number values for video prefix fig1video: [2]\n"
            % info_prefix,
            "%s replacement number values can be used for video prefix fig1video: [3]\n"
            % info_prefix,
        ]
        new_key_map = video.renumber_key_map(prefix, key_map)
        self.assertEqual(new_key_map, expected)
        self.assertEqual(read_log_file_lines(self.log_file), expected_log_file_lines)


class TestTermsFromTitle(unittest.TestCase):
    def test_terms_from_title(self):
        title = (
            "Animation 999 "
            "- Video 2 "
            "- Appendix 3 "
            "- Figure 4 "
            "- Figure Supplement 5 "
            "- Video Supplement 6 "
            "- Videos Supplement 7 "
            "- Supplementary Video 8 "
            "Figure-4-Video 2"
            "- Vidoe 9"
        )
        expected = [
            OrderedDict([("name", "video"), ("number", "999")]),
            OrderedDict([("name", "video"), ("number", "2")]),
            OrderedDict([("name", "app"), ("number", "3")]),
            OrderedDict([("name", "fig"), ("number", "4")]),
            OrderedDict([("name", "video"), ("number", "6")]),
            OrderedDict([("name", "video"), ("number", "7")]),
            OrderedDict([("name", "video"), ("number", "8")]),
            OrderedDict([("name", "fig"), ("number", "4")]),
            OrderedDict([("name", "video"), ("number", "2")]),
        ]

        self.assertEqual(video.terms_from_title(title), expected)

    def test_terms_from_title_empty_string(self):
        title = ""
        expected = []
        self.assertEqual(video.terms_from_title(title), expected)


class TestVideoId(unittest.TestCase):
    def test_video_id(self):
        for test_data in VIDEO_TITLE_EXAMPLES:
            self.assertEqual(
                video.video_id(test_data.get("title")), test_data.get("id")
            )


class TestVideoFilename(unittest.TestCase):
    def test_video_filename(self):
        for test_data in VIDEO_TITLE_EXAMPLES:
            self.assertEqual(
                video.video_filename(
                    test_data.get("title"),
                    test_data.get("upload_file_nm"),
                    test_data.get("article_id"),
                ),
                test_data.get("filename"),
                "title: %s" % test_data.get("title"),
            )



import unittest
from xml.etree import ElementTree
from elifecleaner import utils


class TestPadMsid(unittest.TestCase):
    def test_pad_msid(self):
        self.assertEqual(utils.pad_msid(666), "00666")
        self.assertEqual(utils.pad_msid("666"), "00666")


class TestFileExtension(unittest.TestCase):
    def test_file_extension(self):
        passes = [(None, None), ("elife-70493-sa1-fig1.png", "png")]
        for file_name, expected in passes:
            self.assertEqual(
                utils.file_extension(file_name),
                expected,
            )


class TestInfFileIdentifier(unittest.TestCase):
    "tests for utils.inf_file_identifier()"

    def test_inf_file_identifer(self):
        "identifier portion of an inline-graphic file name"
        inf_file_name = "elife-70493-inf1.png"
        expected = "inf1"
        self.assertEqual(utils.inf_file_identifier(inf_file_name), expected)


class TestMatchControlCharacterEntities(unittest.TestCase):
    def test_match_control_character_entities(self):
        self.assertEqual([], utils.match_control_character_entities(""))
        self.assertEqual(
            ["&#x001C;"], utils.match_control_character_entities("&#x001C;")
        )
        self.assertEqual(
            ["&#x001C;"], utils.match_control_character_entities("aaaa&#x001C;--")
        )
        self.assertEqual(
            ["&#x001E;", "&#x001E;"],
            utils.match_control_character_entities(" &#x001E;--&#x001E;"),
        )


class TestReplaceControlCharacterEntities(unittest.TestCase):
    def test_replace_control_character_entities(self):
        # empty string
        self.assertEqual("", utils.replace_control_character_entities(""))
        # one entity
        self.assertEqual(
            utils.CONTROL_CHARACTER_ENTITY_REPLACEMENT,
            utils.replace_control_character_entities("&#x001C;"),
        )
        # multiple entities
        self.assertEqual(
            utils.CONTROL_CHARACTER_ENTITY_REPLACEMENT * 4,
            utils.replace_control_character_entities("&#x00;&#x001C;&#x001D;&#x001E;"),
        )
        # entity found inside a string
        string_base = "<title>To %snd odd entities.</title>"
        string = string_base % "&#x001D;"
        expected = string_base % utils.CONTROL_CHARACTER_ENTITY_REPLACEMENT
        self.assertEqual(expected, utils.replace_control_character_entities(string))


class TestXlinkHref(unittest.TestCase):
    def test_xlink_href(self):
        image_href = "image.png"
        xml_string = (
            '<inline-graphic xmlns:xlink="http://www.w3.org/1999/xlink" xlink:href="%s"/>'
        ) % image_href
        href = utils.xlink_href(ElementTree.fromstring(xml_string))
        self.assertEqual(href, image_href)


class TestOpenTag(unittest.TestCase):
    def test_open_tag(self):
        tag_name = "italic"
        expected = "<italic>"
        self.assertEqual(utils.open_tag(tag_name), expected)

    def test_open_tag_with_attr(self):
        tag_name = "xref"
        attr = {"id": "sa2fig1", "ref-type": "fig"}
        expected = '<xref id="sa2fig1" ref-type="fig">'
        self.assertEqual(utils.open_tag(tag_name, attr), expected)


class TestRemoveTags(unittest.TestCase):
    "tests for utils.remove_tags()"

    def test_remove_tags(self):
        xml_string = "<root><hr/></root>"
        expected = b"<root />"
        xml_root = ElementTree.fromstring(xml_string)
        result = utils.remove_tags(xml_root, "hr")
        self.assertEqual(ElementTree.tostring(result), expected)

    def test_complicated_remove_tags(self):
        xml_string = "<root><hr/><p><hr/></p></root>"
        expected = b"<root><p /></root>"
        xml_root = ElementTree.fromstring(xml_string)
        result = utils.remove_tags(xml_root, "hr")
        self.assertEqual(ElementTree.tostring(result), expected)



import os
import unittest
from xml.dom import minidom
from xml.etree import ElementTree
from xml.etree.ElementTree import Element
from elifetools import xmlio
from elifecleaner import configure_logging, table, LOGGER
from tests.helpers import (
    delete_files_in_folder,
    read_fixture,
    read_log_file_lines,
    sub_article_xml_fixture,
)


def table_sub_article_xml_fixture():
    "generate XML test fixture for a referee-report"
    article_type = b"referee-report"
    sub_article_id = b"sa1"
    front_stub_xml_string = (
        b"<front-stub>"
        b'<article-id pub-id-type="doi">10.7554/eLife.79713.1.sa1</article-id>'
        b"<title-group>"
        b"<article-title>Reviewer #1 (Public Review):</article-title>"
        b"</title-group>"
        b"</front-stub>"
    )
    body_xml_string = (
        b"<body>"
        b"<p>First paragraph.</p>"
        b"<p><bold>Review table 1.</bold></p>"
        b"<p>Table title. This is the caption for this table that describes what it contains.</p>"
        b'<p> <inline-graphic xlink:href="elife-70493-inf1.png" /> </p>'
        b"<p>Another paragraph with an inline graphic "
        b'<inline-graphic xlink:href="elife-70493-inf2.jpg" /></p>'
        b"</body>"
    )
    return sub_article_xml_fixture(
        article_type, sub_article_id, front_stub_xml_string, body_xml_string
    )


class TestTableInlineGraphicHrefs(unittest.TestCase):
    "tests for table.table_inline_graphic_hrefs()"

    def test_table_inline_graphic_hrefs(self):
        "get a list of xlink:href values from inline-graphic tags to be converted to table-wrap"
        xml_string = (
            b'<sub-article id="sa1" xmlns:xlink="http://www.w3.org/1999/xlink">'
            b"<body>"
            b"<p><bold>Review table 1.</bold></p>"
            b'<p><inline-graphic xlink:href="elife-70493-inf1.png"/></p>'
            b"<p>Next paragraph is not an inline-graphic href.</p>"
            b'<p><inline-graphic xlink:href="elife-70493-inf2.png"/></p>'
            b"</body>"
            b"</sub-article>"
        )
        identifier = "test.zip"
        tag = ElementTree.fromstring(xml_string)
        expected = ["elife-70493-inf1.png"]
        result = table.table_inline_graphic_hrefs(tag, identifier)
        self.assertEqual(result, expected)


class TestTableGraphicHrefs(unittest.TestCase):
    "tests for table.table_graphic_hrefs()"

    def test_table_graphic_hrefs(self):
        "get a list of xlink:href values from table-wrap graphic tags"
        xml_string = (
            b'<sub-article id="sa1" xmlns:xlink="http://www.w3.org/1999/xlink"><body>'
            b'<table-wrap id="sa1table1">\n'
            b"<label>Review table 1.</label>\n"
            b"<caption>\n"
            b"<title>Caption title.</title>\n"
            b"<p>Caption paragraph.</p>\n"
            b"</caption>\n"
            b'<graphic mimetype="image" mime-subtype="jpg"'
            b' xlink:href="elife-95901-sa1-table1.jpg"/>\n'
            b"</table-wrap>\n"
            b"</body></sub-article>"
        )
        identifier = "test.zip"
        tag = ElementTree.fromstring(xml_string)
        expected = ["elife-95901-sa1-table1.jpg"]
        result = table.table_graphic_hrefs(tag, identifier)
        self.assertEqual(result, expected)

    def test_graphic_hrefs_no_match(self):
        "empty list of xlink:href values when there is no graphic tag"
        xml_string = (
            b'<sub-article id="sa1" xmlns:xlink="http://www.w3.org/1999/xlink">'
            b"<body><p/></body>"
            b"</sub-article>"
        )
        identifier = "test.zip"
        tag = ElementTree.fromstring(xml_string)
        expected = []
        result = table.table_graphic_hrefs(tag, identifier)
        self.assertEqual(result, expected)


class TestTransformTable(unittest.TestCase):
    "tests for table.transform_table()"

    def setUp(self):
        self.identifier = "test.zip"
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_referee_report(self):
        "convert tags to a fig in a referee-report fixture"
        # register XML namespaces
        xmlio.register_xmlns()
        xml_string = table_sub_article_xml_fixture()
        sub_article_root = ElementTree.fromstring(xml_string)
        root = table.transform_table(sub_article_root, self.identifier)

        rough_xml_string = ElementTree.tostring(root, "utf-8")
        self.assertTrue("<p>First paragraph.</p>" in rough_xml_string.decode("utf8"))
        self.assertTrue(
            '<inline-graphic xlink:href="elife-70493-inf2.jpg" />'
            in rough_xml_string.decode("utf8")
        )
        self.assertTrue(
            (
                "<body>"
                "<p>First paragraph.</p>"
                '<table-wrap id="sa1table1">'
                "<label>Review table 1.</label>"
                "<caption>"
                "<title>Table title.</title>"
                "<p>This is the caption for this table that describes what it contains.</p>"
                "</caption>"
                '<graphic mimetype="image" mime-subtype="png" xlink:href="elife-70493-sa1-table1.png" />'
                "</table-wrap>"
                "<p>Another paragraph with an inline graphic "
                '<inline-graphic xlink:href="elife-70493-inf2.jpg" />'
                "</p>"
                "</body>"
            )
            in rough_xml_string.decode("utf8")
        )
        block_info_prefix = "INFO elifecleaner:block"
        expected = [
            '%s:%s: %s potential label "Review table 1." in p tag 1 of id sa1\n'
            % (block_info_prefix, "is_p_label", self.identifier),
            "%s:%s: %s label p tag index 1 of id sa1\n"
            % (block_info_prefix, "tag_index_groups", self.identifier),
            "%s:%s: %s no inline-graphic tag found in p tag 2 of id sa1\n"
            % (block_info_prefix, "is_p_inline_graphic", self.identifier),
            "%s:%s: %s only inline-graphic tag found in p tag 3 of id sa1\n"
            % (block_info_prefix, "is_p_inline_graphic", self.identifier),
        ]
        self.assertEqual(read_log_file_lines(self.log_file), expected)


class TestTsvToList(unittest.TestCase):
    "tests for table.tsv_to_list()"

    def test_tsv_to_list(self):
        "test converting tab separated value string to a list"
        tsv_string = read_fixture("85111_table.tsv")
        expected = read_fixture("85111_table.py")
        result = table.tsv_to_list(tsv_string)
        self.assertEqual(result, expected)

    def test_none(self):
        "test None"
        tsv_string = None
        expected = []
        result = table.tsv_to_list(tsv_string)
        self.assertEqual(result, expected)


class TestListToTableXml(unittest.TestCase):
    "tests for table.list_to_table_xml()"

    def setUp(self):
        self.maxDiff = None

    def test_list_to_table_xml(self):
        "test converting a list of rows into an XML table"
        encoding = "utf-8"
        indent = "    "
        table_rows = read_fixture("85111_table.py")
        expected = read_fixture("85111_table.xml")
        result = table.list_to_table_xml(table_rows)
        self.assertTrue(isinstance(result, Element))
        # convert result to a pretty string
        rough_string = ElementTree.tostring(result)
        reparsed = minidom.parseString(rough_string)
        pretty_result = reparsed.toprettyxml(indent, encoding=encoding).decode("utf-8")
        self.assertEqual(pretty_result, expected)

    def test_inline_tag(self):
        "test for inline tags in the table rows"
        table_rows = [["<italic>Italic\n</italic>>"], ["<<bold>&</bold><foo>"]]
        expected = (
            "<table>"
            "<thead>"
            "<tr>"
            "<th><italic>Italic<break /></italic>&gt;</th>"
            "</tr>"
            "</thead>"
            "<tbody>"
            "<tr>"
            "<td>&lt;<bold>&amp;</bold>&lt;foo&gt;</td>"
            "</tr>"
            "</tbody>"
            "</table>"
        )
        result = table.list_to_table_xml(table_rows)
        self.assertTrue(isinstance(result, Element))
        self.assertEqual(ElementTree.tostring(result).decode("utf-8"), expected)



from collections import Counter, OrderedDict
import re
from elifecleaner import LOGGER
from elifecleaner.utils import pad_msid

JOURNAL = "elife"

STRING_TO_TERM_MAP = {
    "animation": "video",
    "video": "video",
    "appendix": "app",
    "figure": "fig",
    "video supplement": "video",
    "videos supplement": "video",
    "supplementary video": "video",
}

IGNORE_TERM = "audio"

# todo!!!
# - raise exception when ignore term is found, or when video term is not found, to distinguish between them
# - enhance regex to handle mixed numeric values like 6A


def video_file_list(files):
    'get a list of file tags from the XML with @file-type="video"'
    return [
        file_data
        for file_data in files
        if isinstance(file_data, dict) and file_data.get("file_type") == "video"
    ]


def collect_video_data(files):
    "from the list of files collect file name and metadata for videos"
    video_files = video_file_list(files)
    video_data = []
    for file_data in video_files:
        data = OrderedDict()
        data["upload_file_nm"] = file_data.get("upload_file_nm")
        for meta in file_data.get("custom_meta"):
            if meta.get("meta_name") == "Title":
                data["title"] = meta.get("meta_value")
                break
        if len(data.keys()) > 1:
            video_data.append(data)
    return video_data


def rename_video_data(video_data, article_id):
    "generate new video filename and id from the video data"
    generated_video_data = []
    video_terms = all_terms_map(video_data)
    for data in video_data:
        video_data_output = OrderedDict()
        terms = video_terms.get(data.get("upload_file_nm"))
        video_data_output["upload_file_nm"] = data.get("upload_file_nm")
        video_data_output["video_id"] = video_id_from_terms(terms)
        video_data_output["video_filename"] = video_filename_from_terms(
            terms, data.get("upload_file_nm"), article_id
        )
        generated_video_data.append(video_data_output)
    return generated_video_data


def video_data_from_files(files, article_id):
    "from a list of files return video data to be used in renaming files and XML"
    video_data = collect_video_data(files)
    return rename_video_data(video_data, article_id)


def all_terms_map(video_data):
    "get a map file name to oterms for all videos"
    term_map = OrderedDict()
    for data in video_data:
        term_map[data.get("upload_file_nm")] = terms_from_title(data.get("title"))
    # todo!! check all title values are non-None

    # check for duplicates
    a_list = [str(term) for term in term_map.values()]
    unique_terms = list({str(term) for term in term_map.values()})
    if len(a_list) != len(unique_terms):
        # handle duplicates
        LOGGER.info("found duplicate video term values")
        # renumber duplicate terms
        term_map = renumber(term_map)

    return term_map


def renumber(term_map):
    "renumber duplicate video terms in the map"
    # detect duplicate video_id strings and collect the keys of the duplicates
    # first generate a map of video file name to video_id values
    key_map = {key: video_id_from_terms(value) for (key, value) in term_map.items()}
    # get a list of duplicates
    duplicate_video_id_counter = Counter([video_id for video_id in key_map.values()])
    duplicates = [key for key, value in duplicate_video_id_counter.items() if value > 1]
    LOGGER.info("duplicate values: %s", duplicates)
    # for each of the duplicates, find all the keys with the same string prefix
    prefix_to_keys_map = {}
    for dupe in duplicates:
        video_id_prefix = dupe.rstrip("1234567890")
        prefix_to_keys_map[video_id_prefix] = [
            key for key, value in key_map.items() if value.startswith(video_id_prefix)
        ]
    # renumber the videos with duplicate numbers
    return renumber_term_map(term_map, prefix_to_keys_map)


def renumber_term_map(term_map, prefix_to_keys_map):
    "replace duplicate number values in the term_map for each having the same prefix"
    for prefix, keys in prefix_to_keys_map.items():
        key_map = {
            key: int(terms[-1].get("number"))
            for key, terms in term_map.items()
            if key in keys
        }
        # assign new number to duplicate number values
        new_key_map = renumber_key_map(prefix, key_map)
        # replace the number values with new ones
        for key, new_number in new_key_map.items():
            old_number = term_map[key][-1].get("number")
            if str(new_number) != old_number:
                LOGGER.info(
                    "replacing number %s with %s for term %s",
                    old_number,
                    new_number,
                    key,
                )
            term_map[key][-1]["number"] = str(new_number)
    return term_map


def renumber_key_map(prefix, key_map):
    "replace duplicate numbers in the video key_map with new numbers"
    all_numbers = [number for key, number in key_map.items()]
    LOGGER.info("number values used for video prefix %s: %s", prefix, all_numbers)
    duplicate_number_counter = Counter(all_numbers)
    duplicate_numbers = [
        key for key, value in duplicate_number_counter.items() if value > 1
    ]
    LOGGER.info(
        "duplicate number values for video prefix %s: %s", prefix, duplicate_numbers
    )
    # list of replacement numbers are those not already in the list starting from 1
    replacement_numbers = [
        number
        for number in range(min(all_numbers), len(all_numbers) + min(all_numbers))
        if number not in all_numbers
    ]
    LOGGER.info(
        "replacement number values can be used for video prefix %s: %s",
        prefix,
        replacement_numbers,
    )
    replacement_index = 0
    numbers_used = []
    for key in key_map.keys():
        number = key_map[key]
        # replace the number if it is a known duplicate and it has already been used at least once
        if int(number) in duplicate_numbers:
            if number in numbers_used:
                new_number = replacement_numbers[replacement_index]
                key_map[key] = new_number
                numbers_used.append(new_number)
                replacement_index += 1
            else:
                numbers_used.append(number)
    return key_map


def terms_from_title(title):
    "from a title string extract video terms and numbers"
    terms = []
    # ignore the value if audio is in the title
    if IGNORE_TERM in title.lower():
        return []
    # convert some punctuation to space for more lenient matching
    for char in ["_", "-"]:
        title = title.replace(char, " ")
    match_pattern = re.compile(r"(\D*?)(\d+)")
    for match in match_pattern.findall(title):
        section_term = match[0].lstrip(" -").strip().lower()
        if section_term in STRING_TO_TERM_MAP:
            term = OrderedDict()
            term["name"] = STRING_TO_TERM_MAP.get(section_term)
            term["number"] = match[1]
            terms.append(term)
    # check video is one of the name values
    if "video" in [term.get("name", "") for term in terms]:
        return terms
    return []


def video_id(title):
    "generate an id attribute for a video from its title string"
    terms = terms_from_title(title)
    return video_id_from_terms(terms)


def video_id_from_terms(terms):
    id_string = ""
    if not terms:
        return None
    for term in terms:
        id_string += "%s%s" % (term.get("name"), term.get("number"))
    return id_string


def video_filename(title, upload_file_nm, article_id, journal=JOURNAL):
    "generate a new file name for a video file"
    terms = terms_from_title(title)
    return video_filename_from_terms(terms, upload_file_nm, article_id, journal)


def video_filename_from_terms(terms, upload_file_nm, article_id, journal=JOURNAL):
    "generate a new file name for a video file using the terms provided"
    if not terms:
        return None
    new_filename_parts = []
    new_filename_parts.append(journal)
    new_filename_parts.append(pad_msid(article_id))
    for term in terms:
        new_filename_parts.append("%s%s" % (term.get("name"), term.get("number")))
    new_filename = "-".join(new_filename_parts)
    # add file extension
    file_extension = upload_file_nm.split(".")[-1]
    new_filename = "%s.%s" % (new_filename, file_extension)
    return new_filename



from xml.etree.ElementTree import Element
from elifecleaner import block, utils


def fig_file_name_identifier(sub_article_id, fig_index):
    "create the unique portion of a fig file name"
    return "%s-fig%s" % (sub_article_id, fig_index)


def fig_id(sub_article_id, fig_index):
    "create an id attribute for a fig tag"
    return "%sfig%s" % (sub_article_id, fig_index)


def fig_file_name(inf_file_name, sub_article_id, fig_index):
    "from inf file name create a new fig file name"
    return inf_file_name.replace(
        utils.inf_file_identifier(inf_file_name),
        "%s-fig%s" % (sub_article_id, fig_index),
    )


def fig_tag_index_groups(body_tag, sub_article_id, identifier):
    "iterate through the tags in body_tag and find groups of tags to be converted to a fig"
    return block.tag_index_groups(body_tag, sub_article_id, "fig", identifier)


def remove_tag_attributes(tag):
    "remove attributes from the tag"
    if not isinstance(tag, Element):
        return
    attribute_names = [name for name in tag.attrib]
    for attrib_name in attribute_names:
        del tag.attrib[attrib_name]


def inline_graphic_hrefs(sub_article_root, identifier):
    "get inline-graphic href values"
    sub_article_id, body_tag = block.sub_article_tag_parts(sub_article_root)
    href_list = []
    if body_tag is not None:
        # match paragraphs with fig data in them and record the tag indexes
        fig_index_groups = fig_tag_index_groups(body_tag, sub_article_id, identifier)
        href_list = block.graphic_href_list(body_tag, fig_index_groups)
    return href_list


def graphic_hrefs(sub_article_root, identifier):
    "get graphic href values"
    sub_article_id, body_tag = block.sub_article_tag_parts(sub_article_root)
    href_list = []
    if body_tag is not None:
        for graphic_tag in body_tag.findall(".//graphic"):
            image_href = utils.xlink_href(graphic_tag)
            if image_href:
                href_list.append(image_href)
    return href_list


def transform_fig_group(body_tag, fig_index, fig_group, sub_article_id):
    "transform one set of p tags into fig tags as specified in the fig_group dict"
    inline_graphic_p_tag = body_tag[fig_group.get("inline_graphic_index")]
    inline_graphic_tag = block.inline_graphic_tag_from_tag(inline_graphic_p_tag)
    image_href = utils.xlink_href(inline_graphic_tag)

    # insert tags into original inline-graphic
    block.set_label_tag(inline_graphic_p_tag, body_tag, fig_group.get("label_index"))

    # caption
    if fig_group.get("caption_index"):
        block.set_caption_tag(
            inline_graphic_p_tag, body_tag, fig_group.get("caption_index")
        )

    # rename the image file
    new_file_name = fig_file_name(image_href, sub_article_id, fig_index)

    # graphic tag
    block.set_graphic_tag(inline_graphic_p_tag, image_href, new_file_name)

    # convert inline-graphic p tag to a fig tag and remove attributes
    inline_graphic_p_tag.tag = "fig"
    inline_graphic_p_tag.set("id", fig_id(sub_article_id, fig_index))

    # delete the old inline-graphic tag
    inline_graphic_p_tag.remove(inline_graphic_tag)

    # remove the old p tags
    if fig_group.get("caption_index"):
        del body_tag[fig_group.get("caption_index")]
    del body_tag[fig_group.get("label_index")]


def transform_fig_groups(body_tag, fig_index_groups, sub_article_id):
    "transform p tags in the body_tag to fig tags as listed in fig_index_groups"
    # transform the fig tags in reverse order
    fig_index = len(fig_index_groups)
    for fig_group in reversed(fig_index_groups):
        transform_fig_group(body_tag, fig_index, fig_group, sub_article_id)
        # decrement the fig index
        fig_index -= 1


def transform_fig(sub_article_root, identifier):
    "transform inline-graphic tags and related p tags into a fig tag"
    sub_article_id, body_tag = block.sub_article_tag_parts(sub_article_root)
    if body_tag is not None:
        # match paragraphs with fig data in them and record the tag indexes
        fig_index_groups = fig_tag_index_groups(body_tag, sub_article_id, identifier)
        transform_fig_groups(body_tag, fig_index_groups, sub_article_id)
    return sub_article_root



import time
from xml.etree.ElementTree import Element, SubElement
from docmaptools import parse as docmap_parse
from elifearticle.article import Affiliation, Contributor, Role
from elifetools import xmlio
from jatsgenerator import build as jats_build
from elifecleaner import LOGGER

# for each ISSN, values for journal-id-type tag text
ISSN_JOURNAL_ID_MAP = {
    "2050-084X": {
        "nlm-ta": "elife",
        "hwp": "eLife",
        "publisher-id": "eLife",
    }
}

# for each ISSN, values for other journal metadata
ISSN_JOURNAL_META_MAP = {
    "2050-084X": {
        "journal-title": "eLife",
        "publisher-name": "eLife Sciences Publications, Ltd",
    }
}


def yield_journal_id_tags(root, journal_id_types):
    "find journal-id tags with matched journal-id-type attribute"
    for journal_id_tag in root.findall("./front/journal-meta/journal-id"):
        if (
            journal_id_tag.get("journal-id-type")
            and journal_id_tag.get("journal-id-type") in journal_id_types
        ):
            yield journal_id_tag


ELOCATION_ID_PRC_TERM = "RP"


def is_xml_prc(root):
    "check if the XML is PRC format by comparing journal-id tag text for a mismatch"
    issn_tag = root.find("./front/journal-meta/issn")
    if issn_tag is not None and issn_tag.text in ISSN_JOURNAL_ID_MAP:
        journal_id_type_map = ISSN_JOURNAL_ID_MAP.get(issn_tag.text)
        # check if any of the journal-id tag values do not match the expected values
        for journal_id_tag in yield_journal_id_tags(root, journal_id_type_map.keys()):
            if journal_id_tag.text != journal_id_type_map.get(
                journal_id_tag.get("journal-id-type")
            ):
                return True
    # also check the elocation-id tag value if the journal meta has already been changed
    elocation_id_tag = root.find(".//front/article-meta/elocation-id")
    if elocation_id_tag is not None:
        if elocation_id_tag.text and elocation_id_tag.text.startswith(
            ELOCATION_ID_PRC_TERM
        ):
            return True
    return False


def transform_journal_id_tags(root, identifier=None):
    "replace file name tags in xml Element with names from file transformations list"
    issn_tag = root.find("./front/journal-meta/issn")
    if issn_tag is not None and issn_tag.text in ISSN_JOURNAL_ID_MAP:
        journal_id_type_map = ISSN_JOURNAL_ID_MAP.get(issn_tag.text)
        for journal_id_tag in yield_journal_id_tags(root, journal_id_type_map.keys()):
            LOGGER.info(
                "%s replacing journal-id tag text of type %s to %s",
                identifier,
                journal_id_tag.get("journal-id-type"),
                journal_id_type_map.get(journal_id_tag.get("journal-id-type")),
            )

            journal_id_tag.text = journal_id_type_map.get(
                journal_id_tag.get("journal-id-type")
            )
    return root


def transform_journal_meta_tag(root, tag_name, tag_path, identifier=None):
    "replace the text value of a tag in the journal meta"
    issn_tag = root.find("./front/journal-meta/issn")
    if issn_tag is not None and issn_tag.text in ISSN_JOURNAL_META_MAP:
        journal_meta_map = ISSN_JOURNAL_META_MAP.get(issn_tag.text)
        journal_title_tag = root.find(tag_path)
        if journal_title_tag is not None and journal_meta_map.get(tag_name):
            LOGGER.info(
                "%s replacing %s tag text to %s",
                identifier,
                tag_name,
                journal_meta_map.get(tag_name),
            )
            journal_title_tag.text = journal_meta_map.get(tag_name)
    return root


def transform_journal_title_tag(root, identifier=None):
    "replace journal-title tag in xml Element with names from file transformations list"
    return transform_journal_meta_tag(
        root,
        "journal-title",
        "./front/journal-meta/journal-title-group/journal-title",
        identifier,
    )


def transform_publisher_name_tag(root, identifier=None):
    "replace publisher-name tag in xml Element with names from file transformations list"
    return transform_journal_meta_tag(
        root,
        "publisher-name",
        "./front/journal-meta/publisher/publisher-name",
        identifier,
    )


def add_prc_custom_meta_tags(root, identifier=None):
    "add custom-meta tag in custom-meta-group"
    article_meta_tag = root.find(".//front/article-meta")
    if article_meta_tag is None:
        LOGGER.warning(
            "%s article-meta tag not found",
            identifier,
        )
        return root
    custom_meta_group_tag = article_meta_tag.find("custom-meta-group")
    if custom_meta_group_tag is None:
        # add the custom-meta-group tag
        custom_meta_group_tag = SubElement(article_meta_tag, "custom-meta-group")
    # add the custom-meta tag
    custom_meta_tag = SubElement(custom_meta_group_tag, "custom-meta")
    custom_meta_tag.set("specific-use", "meta-only")
    meta_name_tag = SubElement(custom_meta_tag, "meta-name")
    meta_name_tag.text = "publishing-route"
    meta_value_tag = SubElement(custom_meta_tag, "meta-value")
    meta_value_tag.text = "prc"
    return root


def elocation_id_from_docmap(docmap_string, version_doi=None, identifier=None):
    "from the docmap get the elocation-id volume"
    LOGGER.info("Parse docmap json")
    d_json = docmap_parse.docmap_json(docmap_string)
    if not d_json:
        LOGGER.warning(
            "%s parsing docmap returned None",
            identifier,
        )
        return None
    elocation_id = docmap_parse.preprint_electronic_article_identifier(
        d_json, version_doi=version_doi, identifier=identifier
    )
    if not elocation_id:
        LOGGER.warning(
            "%s no elocation_id found in the docmap",
            identifier,
        )
        return None
    return elocation_id


def version_doi_from_docmap(docmap_string, identifier=None, published=True):
    "find the latest preprint DOI from docmap"
    doi = None
    LOGGER.info("Parse docmap json")
    d_json = docmap_parse.docmap_json(docmap_string)
    if not d_json:
        LOGGER.warning(
            "%s parsing docmap returned None",
            identifier,
        )
        return doi
    LOGGER.info("Get latest preprint data from the docmap")
    preprint_data = docmap_parse.docmap_latest_preprint(d_json, published=published)
    if not preprint_data:
        LOGGER.warning(
            "%s no preprint data was found in the docmap",
            identifier,
        )
        return doi
    LOGGER.info("Find the doi in the docmap preprint data")
    doi = preprint_data.get("doi")
    if not doi:
        LOGGER.warning(
            "%s did not find doi data in the docmap preprint data",
            identifier,
        )
        return doi
    LOGGER.info("Version DOI from the docmap: %s", doi)
    return doi


# maximum supported verison number to check for non-version DOI values
MAX_VERSION = 999


def next_version_doi(doi, identifier=None):
    "generate the next version DOI value"
    if not doi:
        return None
    doi_base, version = doi.rsplit(".", 1)
    # check for integer
    try:
        int(version)
    except ValueError:
        LOGGER.warning(
            "%s version from DOI could not be converted to int, version %s",
            identifier,
            version,
        )
        return None
    if int(version) > MAX_VERSION:
        LOGGER.warning(
            "%s failed to determine the version from DOI, version %s exceeds MAX_VERSION %s",
            identifier,
            version,
            MAX_VERSION,
        )
        return None
    next_version = int(version) + 1
    next_doi = "%s.%s" % (doi_base, next_version)
    LOGGER.info(
        "%s next version doi, from DOI %s, next DOI %s", identifier, doi, next_doi
    )
    return next_doi


def add_article_id(parent, text, pub_id_type, specific_use=None):
    "add article-id tag"
    # add article-id tag
    article_id_tag = Element("article-id")
    article_id_tag.set("pub-id-type", pub_id_type)
    if specific_use:
        article_id_tag.set("specific-use", specific_use)
    article_id_tag.text = text
    # insert the new tag into the XML after the last article-id tag
    insert_index = 1
    for tag_index, tag in enumerate(parent.findall("*")):
        if tag.tag == "article-id":
            insert_index = tag_index + 1
    parent.insert(insert_index, article_id_tag)


def add_doi(root, doi, specific_use=None, identifier=None):
    "add article-id tag for the doi to article-meta tag"
    article_meta_tag = root.find(".//front/article-meta")
    if article_meta_tag is None:
        LOGGER.warning(
            "%s article-meta tag not found",
            identifier,
        )
        return root
    # add version DOI article-id tag
    add_article_id(article_meta_tag, doi, pub_id_type="doi", specific_use=specific_use)
    return root


def add_version_doi(root, doi, identifier=None):
    "add version article-id tag for the doi to article-meta tag"
    return add_doi(root, doi, specific_use="version", identifier=identifier)


def review_date_from_docmap(docmap_string, identifier=None):
    "find the under-review date for the first preprint from the docmap"
    date_string = None
    LOGGER.info("Parse docmap json")
    d_json = docmap_parse.docmap_json(docmap_string)
    if not d_json:
        LOGGER.warning(
            "%s parsing docmap returned None",
            identifier,
        )
        return date_string
    LOGGER.info("Get first under-review happened date from the docmap")
    date_string = docmap_parse.preprint_review_date(d_json)
    if not date_string:
        LOGGER.warning(
            "%s no under-review happened date was found in the docmap",
            identifier,
        )
    return date_string


def volume_from_docmap(docmap_string, version_doi=None, identifier=None):
    "from the docmap get the volume"
    LOGGER.info("Parse docmap json")
    d_json = docmap_parse.docmap_json(docmap_string)
    if not d_json:
        LOGGER.warning(
            "%s parsing docmap returned None",
            identifier,
        )
        return None
    volume = docmap_parse.preprint_volume(
        d_json, version_doi=version_doi, identifier=identifier
    )
    if volume:
        try:
            volume = int(volume)
        except ValueError:
            LOGGER.warning(
                "%s volume from the docmap could not be cast as int",
                identifier,
            )
            return None
    else:
        LOGGER.warning(
            "%s no volume found in the docmap",
            identifier,
        )
        return None
    return volume


def article_id_from_docmap(docmap_string, version_doi=None, identifier=None):
    "from the docmap get the article_id"
    LOGGER.info("Parse docmap json")
    d_json = docmap_parse.docmap_json(docmap_string)
    if not d_json:
        LOGGER.warning(
            "%s parsing docmap returned None",
            identifier,
        )
        return None
    article_id = docmap_parse.preprint_identifier(
        d_json, version_doi=version_doi, identifier=identifier
    )
    if not article_id:
        LOGGER.warning(
            "%s no article_id found in the docmap",
            identifier,
        )
        return None
    return article_id


def license_from_docmap(docmap_string, version_doi=None, identifier=None):
    "from the docmap get the license"
    LOGGER.info("Parse docmap json")
    d_json = docmap_parse.docmap_json(docmap_string)
    if not d_json:
        LOGGER.warning(
            "%s parsing docmap returned None",
            identifier,
        )
        return None
    license_url = docmap_parse.preprint_license(
        d_json, version_doi=version_doi, identifier=identifier
    )
    if not license_url:
        LOGGER.warning(
            "%s no license found in the docmap",
            identifier,
        )
        return None
    return license_url


def article_categories_from_docmap(docmap_string, version_doi=None, identifier=None):
    "from the docmap get the article category subject disciplines"
    LOGGER.info("Parse docmap json")
    d_json = docmap_parse.docmap_json(docmap_string)
    if not d_json:
        LOGGER.warning(
            "%s parsing docmap returned None",
            identifier,
        )
        return None
    article_categories = docmap_parse.preprint_subject_disciplines(
        d_json, version_doi=version_doi, identifier=identifier
    )
    if not article_categories:
        LOGGER.warning(
            "%s no article_categories found in the docmap",
            identifier,
        )
        return None
    return article_categories


def date_struct_from_string(date_string):
    "parse the date_string into time.struct_time"
    formats = ["%Y-%m-%dT%H:%M:%S.%f%z", "%Y-%m-%dT%H:%M:%S%z", "%Y-%m-%d"]
    for date_format in formats:
        try:
            return time.strptime(date_string, date_format)
        except ValueError:
            LOGGER.info(
                'unable to parse "%s" using format "%s"',
                date_string,
                date_format,
            )
    return None


def add_history_date(root, date_type, date_struct, identifier=None):
    "find or add the history tag and add a date to it"
    article_meta_tag = root.find(".//front/article-meta")
    if article_meta_tag is None:
        LOGGER.warning(
            "%s article-meta tag not found",
            identifier,
        )
        return root
    # look for an existing history tag
    history_tag = article_meta_tag.find("./history")
    # if no history tag, add one
    if history_tag is None:
        # insert the new tag into the XML after the elocation-id tag
        insert_index = 1
        for tag_index, tag in enumerate(article_meta_tag.findall("*")):
            if tag.tag == "elocation-id":
                insert_index = tag_index + 1
        history_tag = Element("history")
        article_meta_tag.insert(insert_index, history_tag)
    # add date tag to the history tag
    date_tag = SubElement(history_tag, "date")
    if date_type:
        date_tag.set("date-type", date_type)
        date_tag.set("iso-8601-date", time.strftime("%Y-%m-%d", date_struct))
    jats_build.set_dmy(date_tag, date_struct)
    return root


def set_article_id(xml_root, article_id, doi, version_doi):
    "add article-id tags"
    # add manuscript ID
    if article_id:
        article_meta_tag = xml_root.find(".//front/article-meta")
        article_id_tag = Element("article-id")
        article_id_tag.set("pub-id-type", "publisher-id")
        article_id_tag.text = str(article_id)
        article_meta_tag.insert(0, article_id_tag)
    # add regular DOI tag
    add_doi(xml_root, doi, specific_use=None, identifier=version_doi)
    # add version DOI tag
    add_version_doi(xml_root, version_doi, version_doi)


def set_volume(root, volume):
    "set volume tag text, add volume tag if not present"
    # find volume tag
    tag = root.find(".//front/article-meta/volume")
    if tag is None:
        article_meta_tag = root.find(".//front/article-meta")
        if article_meta_tag is not None:
            # add volume tag
            tag_index = xmlio.get_first_element_index(article_meta_tag, "elocation-id")
            if tag_index:
                # insert volume tag in the right tag order
                tag = Element("volume")
                article_meta_tag.insert(tag_index - 1, tag)
            else:
                # append to the end of article-meta
                tag = SubElement(article_meta_tag, "volume")
    if tag is not None:
        tag.text = str(volume)


def set_elocation_id(root, elocation_id):
    "set elocation-id tag text, add elocation-id tag if not present"
    # find elocation-id tag
    tag = root.find(".//front/article-meta/elocation-id")
    if tag is None:
        article_meta_tag = root.find(".//front/article-meta")
        if article_meta_tag is not None:
            # add elocation-id tag
            tag_index = xmlio.get_first_element_index(article_meta_tag, "volume")
            if tag_index:
                # insert elocation-id tag in the right tag order
                tag = Element("elocation-id")
                article_meta_tag.insert(tag_index, tag)
            else:
                # append to the end of article-meta
                tag = SubElement(article_meta_tag, "elocation-id")
    if tag is not None:
        tag.text = str(elocation_id)


def set_article_categories(xml_root, display_channel=None, article_categories=None):
    "add tags to article-categories tag"
    article_meta_tag = xml_root.find(".//front/article-meta")
    article_categories_tag = article_meta_tag.find(".//article-categories")
    if article_categories_tag is None:
        # insert the new tag into the XML after the article-id tags and before title-group
        insert_index = None
        for tag_index, tag in enumerate(article_meta_tag.findall("*")):
            if tag.tag == "article-id":
                insert_index = tag_index + 1
            if tag.tag == "title-group":
                insert_index = tag_index
                break
        article_categories_tag = Element("article-categories")
        article_meta_tag.insert(insert_index, article_categories_tag)

    if article_categories_tag is not None and display_channel:
        jats_build.set_display_channel(article_categories_tag, display_channel)

    if article_categories_tag is not None and article_categories:
        for article_category in article_categories:
            jats_build.set_major_subject_area(article_categories_tag, article_category)


def set_license_tag(parent, license_data_dict):
    "add license tag to parent tag"
    if not license_data_dict.get("href"):
        return
    license_tag = SubElement(parent, "license")
    license_tag.set(
        "{http://www.w3.org/1999/xlink}href",
        str(license_data_dict.get("href")),
    )
    ali_license_href_tag = SubElement(
        license_tag, "{http://www.niso.org/schemas/ali/1.0/}license_ref"
    )
    ali_license_href_tag.text = license_data_dict.get("href")
    # license-p tag
    license_p_tag = SubElement(license_tag, "license-p")
    license_p_tag.text = license_data_dict.get("paragraph1")
    license_ext_link_tag = SubElement(license_p_tag, "ext-link")
    license_ext_link_tag.set("ext-link-type", "uri")
    license_ext_link_tag.set(
        "{http://www.w3.org/1999/xlink}href",
        license_data_dict.get("href"),
    )
    license_ext_link_tag.text = license_data_dict.get("name")
    license_ext_link_tag.tail = license_data_dict.get("paragraph2")


def set_permissions(xml_root, license_data_dict, copyright_year, copyright_holder):
    "add license data to permissions tag"
    article_meta_tag = xml_root.find(".//front/article-meta")
    permissions_tag = article_meta_tag.find("./permissions")
    if permissions_tag is None:
        # insert the new tag into the XML after the pub-history or history tag
        insert_index = None
        for tag_index, tag in enumerate(article_meta_tag.findall("*")):
            if tag.tag == "pub-history":
                insert_index = tag_index + 1
                break
            if tag.tag == "history":
                insert_index = tag_index + 1
        permissions_tag = Element("permissions")
        article_meta_tag.insert(insert_index, permissions_tag)
    if license_data_dict:
        if license_data_dict.get("copyright"):
            jats_build.set_copyright_tags(
                permissions_tag, copyright_year, copyright_holder
            )
        ali_free_to_read_tag = SubElement(
            permissions_tag, "{http://www.niso.org/schemas/ali/1.0/}free_to_read"
        )
        set_license_tag(permissions_tag, license_data_dict)


CONTRIB_TYPE_ROLE_MAP = {"editor": "Reviewing Editor", "senior_editor": "Senior Editor"}


def editor_contributors(docmap_string, version_doi):
    "populate Contributor objects with editor data from a docmap"
    editors = []

    data = docmap_parse.docmap_editor_data(docmap_string, version_doi)

    for data_item in data:
        contrib_type = data_item.get("role", "").replace("-", "_")
        roles = []
        aff = None
        if contrib_type:
            editor_role = Role()
            editor_role.text = CONTRIB_TYPE_ROLE_MAP.get(contrib_type)
            roles.append(editor_role)
        if data_item.get("actor"):
            surname = data_item.get("actor").get("surname")
            given_name = " ".join(
                [
                    name_part
                    for name_part in [
                        data_item.get("actor").get("firstName"),
                        data_item.get("actor").get("_middleName"),
                    ]
                    if name_part
                ]
            )
            # format affiliation
            affiliation_data_dict = data_item.get("actor").get("affiliation")
            if affiliation_data_dict:
                aff = Affiliation()
                if (
                    affiliation_data_dict.get("name")
                    and affiliation_data_dict.get("type") == "organization"
                ):
                    aff.institution = (
                        data_item.get("actor").get("affiliation").get("name")
                    )
                if affiliation_data_dict.get("location"):
                    # parse and set city and country values
                    location_parts = affiliation_data_dict.get("location").split(",")
                    if len(location_parts) == 2:
                        aff.city = location_parts[0]
                        aff.country = location_parts[1].lstrip()

        editor = Contributor(contrib_type, surname, given_name)
        editor.roles = roles
        if aff:
            editor.set_affiliation(aff)
        editors.append(editor)

    return editors


def set_editors(parent, editors):
    "set editor contrib tags"
    if not editors:
        return
    # find where to insert a contrib-group tag
    insert_index = None
    for tag_index, tag in enumerate(parent.findall("*")):
        if tag.tag == "contrib-group":
            insert_index = tag_index + 1
    # insert contrib-group tag
    contrib_group_tag = Element("contrib-group")
    if insert_index:
        parent.insert(insert_index, contrib_group_tag)
    else:
        parent.append(contrib_group_tag)
    contrib_group_tag.set("content-type", "section")
    for editor in editors:
        contrib_tag = SubElement(contrib_group_tag, "contrib")
        contrib_tag.set("contrib-type", editor.contrib_type)
        jats_build.set_contrib_name(contrib_tag, editor)
        jats_build.set_contrib_orcid(contrib_tag, editor)
        # role tag
        jats_build.set_contrib_role(contrib_tag, editor.contrib_type, editor)
        # add inline aff tags
        for affiliation in editor.affiliations:
            jats_build.set_aff(
                contrib_tag,
                affiliation,
                editor.contrib_type,
                tail="",
                institution_wrap=True,
            )



from xml.etree.ElementTree import SubElement
from elifecleaner import block, utils
from elifecleaner.fig import remove_tag_attributes


def inline_formula_graphic_hrefs(sub_article_root, identifier):
    "return a list of inline-formula inline-graphic tag xlink:href values"
    body_tag = block.sub_article_tag_parts(sub_article_root)[1]
    current_hrefs = []
    if body_tag is not None:
        for graphic_tag in sub_article_root.findall(".//inline-formula/inline-graphic"):
            image_href = utils.xlink_href(graphic_tag)
            if image_href:
                current_hrefs.append(image_href)
    return current_hrefs


def transform_equations(sub_article_root, identifier):
    "transform inline-graphic tags into disp-formula tags"
    sub_article_id, body_tag = block.sub_article_tag_parts(sub_article_root)
    if body_tag is not None:
        # match paragraphs with data in them and record the tag indexes
        index_groups = disp_formula_tag_index_groups(body_tag, identifier)
        transform_disp_formulas(body_tag, index_groups, sub_article_id)
    return sub_article_root


def extra_equation_count(parent_tag):
    "count tags which are or may be converted to an equation"
    tag_count = 0
    if parent_tag.findall("inline-graphic"):
        # count the inline-graphic tags
        tag_count += len(parent_tag.findall("inline-graphic"))
    elif parent_tag.findall("inline-formula"):
        # count the inline-formula tags
        tag_count += len(parent_tag.findall("inline-formula"))
    elif parent_tag.tag == "disp-formula" and parent_tag.findall("graphic"):
        # count the block formula tags
        tag_count += len(parent_tag.findall("graphic"))
    return tag_count


def disp_formula_tag_index_groups(body_tag, identifier):
    "find p tags which have inline-graphic tags to convert to disp-formula"
    index_groups = []
    tag_id_index = 1
    for tag_index, parent_tag in enumerate(body_tag.iterfind("*")):
        # count the inline-graphic tags to get an id value
        if block.is_p_inline_graphic(
            tag=parent_tag,
            sub_article_id=None,
            p_tag_index=None,
            identifier=identifier,
        ):
            detail = {
                "label_index": None,
                "caption_index": None,
                "inline_graphic_index": tag_index,
                "tag_index": tag_id_index,
            }
            index_groups.append(detail)
            tag_id_index += 1
        else:
            tag_id_index += extra_equation_count(parent_tag)

    return index_groups


def transform_disp_formulas(body_tag, index_groups, sub_article_id):
    "transform p tags in the body_tag to disp-formula tags as listed in index_groups"
    for group in reversed(index_groups):
        index = group.get("tag_index")
        transform_disp_formula(body_tag, index, group, sub_article_id)


def formula_id(sub_article_id, index):
    "create an id attribute for a disp-formula tag"
    return "%sequ%s" % (sub_article_id, index)


def formula_file_name(inf_file_name, sub_article_id, index):
    "create a file name for an equation graphic file"
    return inf_file_name.replace(
        utils.inf_file_identifier(inf_file_name),
        "%s-equ%s" % (sub_article_id, index),
    )


def transform_disp_formula(body_tag, index, group, sub_article_id):
    "transform one set of p tags into disp-formula tags as specified in the group dict"
    inline_graphic_p_tag = body_tag[group.get("inline_graphic_index")]
    inline_graphic_tag = block.inline_graphic_tag_from_tag(inline_graphic_p_tag)
    image_href = utils.xlink_href(inline_graphic_tag)

    # rename the image file
    new_file_name = formula_file_name(image_href, sub_article_id, index)

    # graphic tag
    block.set_graphic_tag(inline_graphic_p_tag, image_href, new_file_name)

    # convert inline-graphic p tag and remove attributes
    inline_graphic_p_tag.tag = "disp-formula"
    inline_graphic_p_tag.set("id", formula_id(sub_article_id, index))

    # delete the old inline-graphic tag
    inline_graphic_p_tag.remove(inline_graphic_tag)


def equation_inline_graphic_hrefs(sub_article_root, identifier):
    "get inline-graphic xlink:href values to be disp-formula"
    body_tag = block.sub_article_tag_parts(sub_article_root)[1]
    href_list = []
    equation_index_groups = []
    if body_tag is not None:
        # find paragraphs to be disp-formula data
        equation_index_groups = disp_formula_tag_index_groups(body_tag, identifier)
        href_list = block.graphic_href_list(body_tag, equation_index_groups)
    return href_list


def formula_graphic_hrefs(sub_article_root, identifier):
    "get disp-formula graphic href values"
    sub_article_id, body_tag = block.sub_article_tag_parts(sub_article_root)
    href_list = []
    if body_tag is not None:
        for graphic_tag in body_tag.findall(".//disp-formula/graphic"):
            image_href = utils.xlink_href(graphic_tag)
            if image_href:
                href_list.append(image_href)
    return href_list


def inline_equation_inline_graphic_hrefs(sub_article_root, identifier):
    "get inline-graphic xlink:href values to be inline-formula"
    body_tag = block.sub_article_tag_parts(sub_article_root)[1]
    href_list = []
    equation_index_groups = []
    if body_tag is not None:
        # find paragraphs with inline-formula data in them and record the tag indexes
        equation_index_groups = inline_formula_tag_index_groups(body_tag, identifier)
        href_list = block.graphic_href_list(body_tag, equation_index_groups)
    return href_list


def transform_inline_equations(sub_article_root, identifier):
    "transform inline-graphic tags into inline-formula tags"
    sub_article_id, body_tag = block.sub_article_tag_parts(sub_article_root)
    if body_tag is not None:
        # match paragraphs with data in them and record the tag indexes
        index_groups = inline_formula_tag_index_groups(body_tag, identifier)
        transform_inline_formulas(body_tag, index_groups, sub_article_id)
    return sub_article_root


def inline_formula_tag_index_groups(body_tag, identifier):
    "find p tags which have inline-graphic tags to convert to inline-formula"
    index_groups = []
    tag_id_index = 1
    for tag_index, parent_tag in enumerate(body_tag.iterfind("*")):
        if parent_tag.find("inline-graphic") is not None and not (
            block.is_p_inline_graphic(
                tag=parent_tag,
                sub_article_id=None,
                p_tag_index=None,
                identifier=identifier,
            )
        ):
            for inline_graphic_tag in parent_tag.findall("inline-graphic"):
                detail = {
                    "label_index": None,
                    "caption_index": None,
                    "inline_graphic_index": tag_index,
                    "tag_index": tag_id_index,
                }

                index_groups.append(detail)
                tag_id_index += 1
        else:
            tag_id_index += extra_equation_count(parent_tag)

    return index_groups


def transform_inline_formulas(body_tag, index_groups, sub_article_id):
    "transform inline-graphic tags to inline-formula tags as listed in index_groups"
    for group in index_groups:
        index = group.get("tag_index")
        transform_inline_formula(body_tag, index, group, sub_article_id)


def transform_inline_formula(body_tag, index, group, sub_article_id):
    "transform one set of p tags into inline-formula tags as specified in the group dict"
    inline_graphic_p_tag = body_tag[group.get("inline_graphic_index")]
    inline_graphic_tag = block.inline_graphic_tag_from_tag(inline_graphic_p_tag)
    image_href = utils.xlink_href(inline_graphic_tag)

    # rename the inline-graphic tag to inline-formula
    remove_tag_attributes(inline_graphic_tag)
    inline_graphic_tag.tag = "inline-formula"
    inline_graphic_tag.set("id", formula_id(sub_article_id, index))

    # add inline-graphic tag
    new_inline_graphic_tag = SubElement(inline_graphic_tag, "inline-graphic")
    new_file_name = formula_file_name(image_href, sub_article_id, index)
    new_inline_graphic_tag.set("{http://www.w3.org/1999/xlink}href", new_file_name)
